@@ -1,7 +1,7 @@
 #!/bin/bash
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -18,6 +18,7 @@ RUN_INSTALLER=${RUN_INSTALLER:-1}
 DO_MSI=${DO_MSI:-0}
 DO_SIGN=${DO_SIGN:-1}
 BUNDLE_PUTTY=${BUNDLE_PUTTY:-1}
+BUNDLE_OPENSSH=${BUNDLE_OPENSSH:-1}
 BUNDLE_OPENSSL=${BUNDLE_OPENSSL:-1}
 
 PYTHON=${PYTHON:-python2}
@@ -202,7 +203,7 @@ fi
 if [ "${DO_TESTS}" == "1" ]; then
 	echo "* Running unit tests"
 	UNITTEST_LOG="win32/unittest.log"
-	PYTHONPATH=.:./unittests ${PYTHON} ./unittests/unit/run.py >& ${UNITTEST_LOG}
+	PYTHONPATH=.:./unittests XPRA_COMMAND="./scripts/xpra" ${PYTHON} ./unittests/unit/run.py >& ${UNITTEST_LOG}
 	if [ "$?" != "0" ]; then
 		echo "ERROR: unittests have failed, see ${UNITTEST_LOG}:"
 		tail -n 20 "${UNITTEST_LOG}"
@@ -210,10 +211,6 @@ if [ "${DO_TESTS}" == "1" ]; then
 	fi
 fi
 
-# For building Python 3.x Sound sub-app (broken because of cx_Freeze bugs)
-#echo "* Building Python 3.4 Cython modules (see win32/Python${PYTHON_MAJOR_VERSION}-build.log)"
-#${PYTHON} ./setup.py build_ext ${BUILD_OPTIONS} --inplace >& win32/Python${PYTHON_MAJOR_VERSION}-build.log
-
 echo "* Generating installation directory"
 CX_FREEZE_LOG="win32/cx_freeze-install.log"
 ${PYTHON} ./setup.py install_exe ${BUILD_OPTIONS} --install=${DIST} >& ${CX_FREEZE_LOG}
@@ -245,6 +242,7 @@ for x in openblas gfortran quadmath; do
 	mv -f ./lib/numpy/core/lib$x*.dll ./lib/
 	mv -f ./lib/numpy/linalg/lib$x*.dll ./lib/
 done
+mv lib/nacl/libsodium*dll ./lib/
 #gstreamer uses its own lib dir, so this does not belong in the root:
 mv ./libgst*.dll ./lib/gstreamer-1.0/
 #but the main gstreamer lib does:
@@ -259,13 +257,17 @@ if [ "${PYTHON_MAJOR_VERSION}" == "3" ]; then
 	pushd lib > /dev/null
 else
 	mv lib/PIL/*dll ./lib/
-	mv lib/*dll ./ > /dev/null
-	pushd .
+	mv lib/*dll ./
+	pushd . > /dev/null
 fi
 #remove all the pointless duplication:
 for x in `ls *dll`; do
 	find ./ -mindepth 2 -name "${x}" -exec rm {} \;
 done
+#remove empty directories:
+rmdir xpra/*/*/* 2> /dev/null
+rmdir xpra/*/* 2> /dev/null
+rmdir xpra/* 2> /dev/null
 popd > /dev/null
 popd > /dev/null
 
@@ -291,9 +293,19 @@ if [ "${BUNDLE_PUTTY}" == "1" ]; then
 	#rsync -rplogt "${TORTOISESVN}/bin/"*dll "${DIST}/"
 fi
 
+if [ "${BUNDLE_OPENSSH}" == "1" ]; then
+	echo "* Adding OpenSSH"
+	cp -fn "/usr/bin/ssh.exe" "${DIST}/"
+	cp -fn "/usr/bin/sshpass.exe" "${DIST}/"
+	cp -fn "/usr/bin/ssh-keygen.exe" "${DIST}/"
+	for x in 2.0 gcc_s crypto z gssapi asn1 com_err roken crypt heimntlm krb5 heimbase wind hx509 hcrypto sqlite3; do
+		cp -fn /usr/bin/msys-$x*.dll "${DIST}/"
+	done
+fi
+
 if [ "${BUNDLE_OPENSSL}" == "1" ]; then
 	cp -fn "${MINGW_PREFIX}/bin/openssl.exe" "${DIST}/"
-	#use the old filename so we don't have to change the xpra.iss and the py2exe+MSVC build system:
+	#use the old filename so we don't have to change the xpra.iss and the setup.py build system:
 	cp -fn "${MINGW_PREFIX}/ssl/openssl.cnf" "${DIST}/openssl.cfg"
 fi
 
