@@ -19,16 +19,17 @@ mouselog = Logger("mouse")
 cryptolog = Logger("crypto")
 bandwidthlog = Logger("bandwidth")
 
-from xpra.scripts.config import InitExit, parse_with_unit
+from xpra.scripts.config import InitExit
 from xpra.child_reaper import getChildReaper, reaper_cleanup
 from xpra.net import compression
 from xpra.net.protocol import Protocol, sanity_checks
 from xpra.net.net_util import get_network_caps
-from xpra.net.crypto import crypto_backend_init, get_iterations, get_iv, get_salt, choose_padding, gendigest, \
+from xpra.net.digest import get_salt, gendigest
+from xpra.net.crypto import crypto_backend_init, get_iterations, get_iv, choose_padding, \
     ENCRYPTION_CIPHERS, ENCRYPT_FIRST_PACKET, DEFAULT_IV, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING, DEFAULT_PADDING, ALL_PADDING_OPTIONS, PADDING_OPTIONS
 from xpra.version_util import get_version_info, XPRA_VERSION
 from xpra.platform.info import get_name
-from xpra.os_util import get_machine_id, get_user_uuid, load_binary_file, SIGNAMES, PYTHON3, strtobytes, bytestostr, hexstr, monotonic_time, osexpand, BITS, WIN32, OSX
+from xpra.os_util import get_machine_id, get_user_uuid, load_binary_file, SIGNAMES, PYTHON3, strtobytes, bytestostr, hexstr, monotonic_time, osexpand, use_tty, BITS, WIN32, OSX
 from xpra.util import flatten_dict, typedict, updict, repr_ellipsized, nonl, std, envbool, envint, disconnect_is_an_error, dump_all_frames, engs, csv, obsc, first_time
 from xpra.client.mixins.serverinfo_mixin import ServerInfoMixin
 from xpra.client.mixins.fileprint_mixin import FilePrintMixin
@@ -47,8 +48,7 @@ ALLOW_LOCALHOST_PASSWORDS = envbool("XPRA_ALLOW_LOCALHOST_PASSWORDS", True)
 DETECT_LEAKS = envbool("XPRA_DETECT_LEAKS", False)
 LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
 MOUSE_DELAY = envint("XPRA_MOUSE_DELAY", 0)
-AUTO_BANDWIDTH_PCT = envint("XPRA_AUTO_BANDWIDTH_PCT", 80)
-assert AUTO_BANDWIDTH_PCT>1 and AUTO_BANDWIDTH_PCT<=100, "invalid value for XPRA_AUTO_BANDWIDTH_PCT: %i" % AUTO_BANDWIDTH_PCT
+
 
 
 """ Base class for Xpra clients.
@@ -103,11 +103,11 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.password_file = ()
         self.password_index = 0
         self.password_sent = False
-        self.bandwidth_limit = 0
         self.encryption = None
         self.encryption_keyfile = None
         self.server_padding_options = [DEFAULT_PADDING]
         self.server_client_shutdown = True
+        self.server_compressors = []
         #protocol stuff:
         self._protocol = None
         self._priority_packets = []
@@ -126,7 +126,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.init_packet_handlers()
         sanity_checks()
 
-    def init(self, opts):
+    def init(self, opts, _extra_args=[]):
         if self._init_done:
             #the gtk client classes can inherit this method
             #from multiple parents, skip initializing twice
@@ -139,8 +139,6 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.username = opts.username
         self.password = opts.password
         self.password_file = opts.password_file
-        self.bandwidth_limit = parse_with_unit("bandwidth-limit", opts.bandwidth_limit)
-        bandwidthlog("init bandwidth_limit=%s", self.bandwidth_limit)
         self.encryption = opts.encryption or opts.tcp_encryption
         if self.encryption:
             crypto_backend_init()
@@ -160,14 +158,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
             log.warn("Warning: unknown challenge handler '%s'", ch_name)
         if DETECT_LEAKS:
             from xpra.util import detect_leaks
-            detailed = []
-            #example: warning, uses ugly direct import:
-            #try:
-            #    from xpra.x11.bindings.ximage import XShmImageWrapper       #@UnresolvedImport
-            #    detailed.append(XShmImageWrapper)
-            #except:
-            #    pass
-            print_leaks = detect_leaks(log, detailed)
+            print_leaks = detect_leaks()
             self.timeout_add(10*1000, print_leaks)
 
 
@@ -181,6 +172,15 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         raise Exception("override me!")
 
 
+    def may_notify(self, nid, summary, body, *args, **kwargs):
+        notifylog = Logger("notify")
+        notifylog("may_notify(%s, %s, %s, %s, %s)", nid, summary, body, args, kwargs)
+        log.info("%s", summary)
+        if body:
+            for x in body.splitlines():
+                log.info("%s", x)
+
+
     def handle_deadly_signal(self, signum, _frame=None):
         sys.stderr.write("\ngot deadly signal %s, exiting\n" % SIGNAMES.get(signum, signum))
         sys.stderr.flush()
@@ -237,6 +237,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         self.timeout_add(1000, self.quit, exit_code)
 
     def exit(self):
+        log("XpraClientBase.exit() calling %s", sys.exit)
         sys.exit()
 
 
@@ -260,7 +261,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
                 })
         else:
             self._protocol = Protocol(self.get_scheduler(), conn, self.process_packet, self.next_packet)
-        for x in ("keymap-changed", "server-settings", "logging", "input-devices"):
+        for x in (b"keymap-changed", b"server-settings", b"logging", b"input-devices"):
             self._protocol.large_packets.append(x)
         self._protocol.set_compression_level(self.compression_level)
         self._protocol.receive_aliases.update(self._aliases)
@@ -275,7 +276,8 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         process = getattr(conn, "process", None)        #ie: ssh is handled by anotherprocess
         if process:
             proc, name, command = process
-            getChildReaper().add_process(proc, name, command, ignore=True, forget=False)
+            if proc:
+                getChildReaper().add_process(proc, name, command, ignore=True, forget=False)
         netlog("setup_connection(%s) protocol=%s", conn, self._protocol)
 
     def _process_udp_control(self, packet):
@@ -396,24 +398,6 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         mid = get_machine_id()
         if mid:
             capabilities["machine_id"] = mid
-        #get socket speed if we have it:
-        pinfo = self._protocol.get_info()
-        netlog("protocol info=%s", pinfo)
-        socket_speed = pinfo.get("socket", {}).get("speed")
-        if socket_speed:
-            capabilities["connection-data"] = {"speed" : socket_speed}
-        bandwidth_limit = self.bandwidth_limit
-        bandwidthlog("bandwidth-limit setting=%s, socket-speed=%s", self.bandwidth_limit, socket_speed)
-        if bandwidth_limit is None:
-            if socket_speed:
-                #auto: use 80% of socket speed if we have it:
-                bandwidth_limit = socket_speed*AUTO_BANDWIDTH_PCT//100 or 0
-            else:
-                bandwidth_limit = 0
-        bandwidthlog("bandwidth-limit capability=%s", bandwidth_limit)
-        if bandwidth_limit>0:
-            capabilities["bandwidth-limit"] = bandwidth_limit
-
         if self.encryption:
             assert self.encryption in ENCRYPTION_CIPHERS
             iv = get_iv()
@@ -668,8 +652,8 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         return self.do_process_challenge_prompt(packet, prompt)
 
     def do_process_challenge_prompt(self, packet, prompt="password"):
-        authlog("do_process_challenge_prompt() isatty=%s", sys.stdin.isatty())
-        if sys.stdin.isatty() and not os.environ.get("MSYSCON"):
+        authlog("do_process_challenge_prompt() use_tty=%s", use_tty())
+        if use_tty():
             import getpass
             authlog("stdin isatty, using password prompt")
             password = getpass.getpass("%s :" % self.get_challenge_prompt(prompt))
@@ -853,7 +837,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
         try:
             from xpra.net.bytestreams import pretty_socket
             conn = self._protocol._conn
-            text += " for %s server %s" % (conn.socktype, pretty_socket(conn.remote))
+            text += " for user '%s',\n connecting to %s server %s" % (self.username, conn.socktype, pretty_socket(conn.remote))
         except:
             pass
         return text
@@ -994,6 +978,7 @@ class XpraClientBase(ServerInfoMixin, FilePrintMixin):
             if not c.parse_server_capabilities(self):
                 return False
         self.server_client_shutdown = self.server_capabilities.boolget("client-shutdown", True)
+        self.server_compressors = self.server_capabilities.strlistget("compressors", ["zlib"])
         return True
 
     def parse_network_capabilities(self):
