@@ -1,4 +1,4 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
@@ -11,19 +11,20 @@ gdk = import_gdk()
 glib = import_glib()
 import sys
 import datetime
+from collections import deque
 
 from xpra.version_util import XPRA_VERSION
 from xpra.os_util import bytestostr, strtobytes, get_linux_distribution, monotonic_time
 from xpra.util import prettify_plug_name, typedict, csv, engs
 from xpra.gtk_common.graph import make_graph_pixmap
-from collections import deque
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec, std_unit
 from xpra.scripts.config import python_platform
-from xpra.log import Logger
+from xpra.client import mixin_features
 from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, \
                         TableBuilder, imagebutton, scaled_image, get_preferred_size, get_gtk_version_info, \
                         RELIEF_NONE, RELIEF_NORMAL, EXPAND, FILL, WIN_POS_CENTER
-from xpra.net.protocol import get_network_caps
+from xpra.net.net_util import get_network_caps
+from xpra.log import Logger
 log = Logger("info")
 
 N_SAMPLES = 20      #how many sample points to show on the graphs
@@ -75,8 +76,11 @@ def dictlook(d, k, fallback=None):
     v = d.get(k)
     if v is not None:
         return v
-    parts = strtobytes(k).split(b".")
+    parts = (b"client."+strtobytes(k)).split(b".")
     v = newdictlook(d, parts, fallback)
+    if v is None:
+        parts = strtobytes(k).split(b".")
+        v = newdictlook(d, parts, fallback)        
     if v is None:
         return fallback
     return v
@@ -86,7 +90,12 @@ def newdictlook(d, parts, fallback=None):
     v = d
     for p in parts:
         try:
-            v = v.get(p) or v.get(bytestostr(p))
+            newv = v.get(p)
+            if newv is None:
+                newv = v.get(strtobytes(p))
+                if newv is None:
+                    return fallback
+            v = newv
         except:
             return fallback
     return v
@@ -109,9 +118,9 @@ class SessionInfo(gtk.Window):
         self.is_closed = False
         self.get_pixbuf = get_pixbuf
         if not self.session_name or self.session_name=="Xpra":
-            title = "Session Info"
+            title = u"Session Info"
         else:
-            title = "%s: Session Info" % self.session_name
+            title = u"%s: Session Info" % self.session_name
         self.set_title(title)
         self.set_destroy_with_parent(True)
         self.set_resizable(True)
@@ -189,8 +198,8 @@ class SessionInfo(gtk.Window):
                                  slabel(make_datetime(server_info("build_date", "build.date"), server_info("build.time"))))
         gtk_version_info = get_gtk_version_info()
         def client_vinfo(prop, fallback="unknown"):
-            k = "%s.version" % prop
-            return slabel(make_version_str(dictlook(gtk_version_info, k, fallback)))
+            s = make_version_str(newdictlook(gtk_version_info, (prop, "version"), fallback))
+            return slabel(s)
         def server_vinfo(prop):
             k = "%s.version" % prop
             return slabel(server_version_info(k))
@@ -207,11 +216,11 @@ class SessionInfo(gtk.Window):
         try:
             from xpra.sound.wrapper import query_sound
             props = query_sound()
-        except Exception as e:
-            log("cannot load sound information: %s", e)
-            props = {}
-        gst_version = props.get(b"gst.version", "")
-        pygst_version = props.get(b"pygst.version", "")
+        except Exception:
+            log("cannot load sound information: %s", exc_info=True)
+            props = typedict()
+        gst_version = props.strlistget("gst.version")
+        pygst_version = props.strlistget("pygst.version")
         tb.new_row("GStreamer", slabel(make_version_str(gst_version)), slabel(server_version_info("sound.gst.version")))
         tb.new_row("pygst", slabel(make_version_str(pygst_version)), slabel(server_version_info("sound.pygst.version")))
         tb.new_row("OpenGL", slabel(make_version_str(self.client.opengl_props.get("opengl", "n/a"))), slabel(server_version_info("opengl.opengl")))
@@ -298,7 +307,7 @@ class SessionInfo(gtk.Window):
         tb, _ = self.table_tab("connect.png", "Connection", self.populate_connection)
         if self.connection:
             tb.new_row("Server Endpoint", slabel(self.connection.target))
-        if self.client.server_display:
+        if mixin_features.display and self.client.server_display:
             tb.new_row("Server Display", slabel(prettify_plug_name(self.client.server_display)))
         hostname = scaps.strget("hostname")
         if hostname:
@@ -353,9 +362,9 @@ class SessionInfo(gtk.Window):
         self.client_latency_labels = maths_labels()
         tb.add_row(slabel("Client Latency (ms)", "The time it takes for the client to respond to pings, as measured by the server"),
                    *self.client_latency_labels)
-        if self.client.windows_enabled:
+        if mixin_features.windows and self.client.windows_enabled:
             self.batch_labels = maths_labels()
-            tb.add_row(slabel("Batch Delay (ms)", "How long the server waits for new screen updates to accumulate before processing them"),
+            tb.add_row(slabel("Batch Delay (MPixels / ms)", "How long the server waits for new screen updates to accumulate before processing them"),
                        *self.batch_labels)
             self.damage_labels = maths_labels()
             tb.add_row(slabel("Damage Latency (ms)", "The time it takes to compress a frame and pass it to the OS network layer"),
@@ -430,7 +439,7 @@ class SessionInfo(gtk.Window):
         self.add(self.tab_box)
         if not is_gtk3():
             self.set_geometry_hints(self.tab_box)
-        def window_deleted(*args):
+        def window_deleted(*_args):
             self.is_closed = True
         self.connect('delete_event', window_deleted)
         self.show_tab(self.tabs[0][2])
@@ -440,7 +449,7 @@ class SessionInfo(gtk.Window):
         self.populate_all()
         glib.timeout_add(1000, self.populate)
         glib.timeout_add(100, self.populate_tab)
-        if SHOW_SOUND_STATS:
+        if mixin_features.audio and SHOW_SOUND_STATS:
             glib.timeout_add(100, self.populate_sound_stats)
         add_close_accel(self, self.destroy)
 
@@ -462,7 +471,7 @@ class SessionInfo(gtk.Window):
 
     def add_tab(self, icon_filename, title, populate_cb, contents):
         icon = self.get_pixbuf(icon_filename)
-        def show_tab(*args):
+        def show_tab(*_args):
             self.show_tab(contents)
         button = imagebutton(title, icon, clicked_callback=show_tab)
         button.connect("clicked", show_tab)
@@ -539,7 +548,7 @@ class SessionInfo(gtk.Window):
             icon = self.get_pixbuf("unticked-small.png")
         image.set_from_pixbuf(icon)
 
-    def populate_sound_stats(self, *args):
+    def populate_sound_stats(self, *_args):
         #runs every 100ms
         if self.is_closed:
             return False
@@ -555,7 +564,7 @@ class SessionInfo(gtk.Window):
                 self.sound_out_queue_max.append(intlookup("queue.max"))
         return not self.is_closed
 
-    def populate(self, *args):
+    def populate(self, *_args):
         if self.is_closed or not self.connection:
             return False
         self.client.send_ping()
@@ -566,16 +575,17 @@ class SessionInfo(gtk.Window):
         #record bytecount every second:
         self.net_in_bytecount.append(self.connection.input_bytecount)
         self.net_out_bytecount.append(self.connection.output_bytecount)
-        if SHOW_SOUND_STATS:
+        if mixin_features.audio and SHOW_SOUND_STATS:
             if self.client.sound_in_bytecount>0:
                 self.sound_in_bitcount.append(self.client.sound_in_bytecount * 8)
             if self.client.sound_out_bytecount>0:
                 self.sound_out_bitcount.append(self.client.sound_out_bytecount * 8)
 
-        #count pixels in the last second:
-        since = monotonic_time()-1
-        decoded = [0]+[pixels for _,t,pixels in self.client.pixel_counter if t>since]
-        self.pixel_in_data.append(sum(decoded))
+        if mixin_features.windows:
+            #count pixels in the last second:
+            since = monotonic_time()-1
+            decoded = [0]+[pixels for _,t,pixels in self.client.pixel_counter if t>since]
+            self.pixel_in_data.append(sum(decoded))
         #update latency values
         #there may be more than one record for each second
         #so we have to average them to prevent the graph from "jumping":
@@ -612,10 +622,10 @@ class SessionInfo(gtk.Window):
             addavg(self.avg_batch_delay, "batch.delay")
             addavg(self.avg_damage_out_latency, "damage.out_latency")
             if len(self.client.server_ping_latency)>0 and len(self.client.client_ping_latency)>0:
-                spl = [1000.0*x for _,x in list(self.client.server_ping_latency)]
-                cpl = [1000.0*x for _,x in list(self.client.client_ping_latency)]
+                spl = [1000.0*x for _,x in tuple(self.client.server_ping_latency)]
+                cpl = [1000.0*x for _,x in tuple(self.client.client_ping_latency)]
                 self.avg_ping_latency.append(sum(spl+cpl)/len(spl+cpl))
-            if len(self.client.pixel_counter)>0:
+            if mixin_features.windows and len(self.client.pixel_counter)>0:
                 tsize = 0
                 ttime = 0
                 for start_time, end_time, size in self.client.pixel_counter:
@@ -638,7 +648,7 @@ class SessionInfo(gtk.Window):
         self.avg_decoding_latency = deque(maxlen=N_SAMPLES+4)
         self.avg_total = deque(maxlen=N_SAMPLES+4)
 
-    def populate_tab(self, *args):
+    def populate_tab(self, *_args):
         if self.is_closed:
             return False
         #now re-populate the tab we are seeing:
@@ -677,9 +687,11 @@ class SessionInfo(gtk.Window):
         self.opengl_buffering.set_text(" ".join(info))
 
     def show_window_renderers(self):
+        if not mixin_features.windows:
+            return
         wr = []
         renderers = {}
-        for wid, window in list(self.client._id_to_window.items()):
+        for wid, window in tuple(self.client._id_to_window.items()):
             renderers.setdefault(window.get_backing_class(), []).append(wid)
         for bclass, windows in renderers.items():
             wr.append("%s (%i)" % (bclass.__name__.replace("Backing", ""), len(windows)))
@@ -687,15 +699,21 @@ class SessionInfo(gtk.Window):
 
     def populate_features(self):
         size_info = ""
-        if self.client.server_actual_desktop_size:
-            w,h = self.client.server_actual_desktop_size
-            size_info = "%sx%s" % (w,h)
-            if self.client.server_randr and self.client.server_max_desktop_size:
-                size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
-        self.bool_icon(self.server_randr_icon, self.client.server_randr)
+        if mixin_features.windows:
+            if self.client.server_actual_desktop_size:
+                w,h = self.client.server_actual_desktop_size
+                size_info = "%sx%s" % (w,h)
+                if self.client.server_randr and self.client.server_max_desktop_size:
+                    size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
+                self.bool_icon(self.server_randr_icon, self.client.server_randr)
+        else:
+            size_info = "unknown"
+            unknown = self.get_pixbuf("unknown.png")
+            if unknown:
+                self.server_randr_icon.set_from_pixbuf(unknown)
         self.server_randr_label.set_text("%s" % size_info)
         root_w, root_h = self.client.get_root_size()
-        if self.client.xscale!=1 or self.client.yscale!=1:
+        if mixin_features.windows and (self.client.xscale!=1 or self.client.yscale!=1):
             sw, sh = self.client.cp(root_w, root_h)
             display_info = "%ix%i (scaled from %ix%i)" % (sw, sh, root_w, root_h)
         else:
@@ -729,10 +747,11 @@ class SessionInfo(gtk.Window):
             if not enabled:
                 return "n/a"
             return ", ".join(codecs or [])
-        self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
-        self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
-        self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
-        self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
+        if mixin_features.audio:
+            self.server_speaker_codecs_label.set_text(codec_info(scaps.boolget("sound.send", False), scaps.strlistget("sound.encoders", [])))
+            self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
+            self.server_microphone_codecs_label.set_text(codec_info(scaps.boolget("sound.receive", False), scaps.strlistget("sound.decoders", [])))
+            self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
         def encliststr(v):
             v = list(v)
             try:
@@ -742,7 +761,10 @@ class SessionInfo(gtk.Window):
             return csv(sorted(v))
         se = scaps.strlistget("encodings.core", scaps.strlistget("encodings"))
         self.server_encodings_label.set_text(encliststr(se))
-        self.client_encodings_label.set_text(encliststr(self.client.get_core_encodings()))
+        if mixin_features.encoding:
+            self.client_encodings_label.set_text(encliststr(self.client.get_core_encodings()))
+        else:
+            self.client_encodings_label.set_text("n/a")
 
         def get_encoder_list(caps):
             from xpra.net import packet_encoding
@@ -779,32 +801,33 @@ class SessionInfo(gtk.Window):
         self.output_packets_label.set_text(std_unit_dec(p.output_packetcount))
         self.output_bytes_label.set_text(std_unit_dec(c.output_bytecount))
 
-        def get_sound_info(supported, prop):
-            if not supported:
-                return {"state" : "disabled"}
-            if prop is None:
-                return {"state" : "inactive"}
-            return prop.get_info()
-        def set_sound_info(label, details, supported, prop):
-            d = typedict(get_sound_info(supported, prop))
-            state = d.strget("state", "")
-            codec_descr = d.strget("codec") or d.strget("codec_description")
-            container_descr = d.strget("container_description", "")
-            if state=="active" and codec_descr:
-                if codec_descr.find(container_descr)>=0:
-                    descr = codec_descr
-                else:
-                    descr = csv(x for x in (codec_descr, container_descr) if x)
-                state = "%s: %s" % (state, descr)
-            label.set_text(state)
-            if details:
-                s = ""
-                bitrate = d.intget("bitrate", 0)
-                if bitrate>0:
-                    s = "%sbit/s" % std_unit(bitrate)
-                details.set_text(s)
-        set_sound_info(self.speaker_label, self.speaker_details, self.client.speaker_enabled, self.client.sound_sink)
-        set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
+        if mixin_features.audio:
+            def get_sound_info(supported, prop):
+                if not supported:
+                    return {"state" : "disabled"}
+                if prop is None:
+                    return {"state" : "inactive"}
+                return prop.get_info()
+            def set_sound_info(label, details, supported, prop):
+                d = typedict(get_sound_info(supported, prop))
+                state = d.strget("state", "")
+                codec_descr = d.strget("codec") or d.strget("codec_description")
+                container_descr = d.strget("container_description", "")
+                if state=="active" and codec_descr:
+                    if codec_descr.find(container_descr)>=0:
+                        descr = codec_descr
+                    else:
+                        descr = csv(x for x in (codec_descr, container_descr) if x)
+                    state = "%s: %s" % (state, descr)
+                label.set_text(state)
+                if details:
+                    s = ""
+                    bitrate = d.intget("bitrate", 0)
+                    if bitrate>0:
+                        s = "%sbit/s" % std_unit(bitrate)
+                    details.set_text(s)
+            set_sound_info(self.speaker_label, self.speaker_details, self.client.speaker_enabled, self.client.sound_sink)
+            set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
 
         self.connection_type_label.set_text(c.socktype)
         protocol_info = p.get_info()
@@ -847,6 +870,15 @@ class SessionInfo(gtk.Window):
         return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
 
     def all_values_from_info(self, *window_props):
+        #newer (2.4 and later) servers can just give us the value directly:
+        for window_prop in window_props:
+            v = dictlook(self.client.server_last_info, "client.%s" % window_prop)
+            if v is not None:
+                v = typedict(v)
+                getv = v.intget
+                return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
+
+        #legacy servers: sum up the values for all the windows found
         def avg(values):
             if not values:
                 return ""
@@ -858,7 +890,8 @@ class SessionInfo(gtk.Window):
             for wid in self.client._window_to_id.values():
                 for window_prop in window_props:
                     #Warning: this is ugly...
-                    v = self.client.server_last_info.get("window[%s].%s.%s" % (wid, window_prop, suffix))
+                    proppath = "window[%s].%s.%s" % (wid, window_prop, suffix)
+                    v = self.client.server_last_info.get(proppath)
                     if v is None:
                         wprop = window_prop.split(".")              #ie: "encoding.speed" -> ["encoding", "speed"]
                         newpath = ["window", wid]+wprop+[suffix]    #ie: ["window", 1, "encoding", "speed", "cur"]
@@ -866,10 +899,12 @@ class SessionInfo(gtk.Window):
                     if v is not None:
                         values.append(v)
                         break
+            if not values:
+                return ""
             try:
                 return op(values)
             except:
-                #no values?
+                log("%s(%s)", op, values, exc_info=True)
                 return ""
         return getv("cur", avg), getv("min", min), getv("avg", avg), getv("90p", avg), getv("max", max)
 
@@ -904,12 +939,12 @@ class SessionInfo(gtk.Window):
             setall(labels, rounded_values)
 
         if len(self.client.server_ping_latency)>0:
-            spl = [1000.0*x for _,x in list(self.client.server_ping_latency)]
+            spl = [1000.0*x for _,x in tuple(self.client.server_ping_latency)]
             setlabels(self.server_latency_labels, spl)
         if len(self.client.client_ping_latency)>0:
-            cpl = [1000.0*x for _,x in list(self.client.client_ping_latency)]
+            cpl = [1000.0*x for _,x in tuple(self.client.client_ping_latency)]
             setlabels(self.client_latency_labels, cpl)
-        if self.client.windows_enabled:
+        if mixin_features.windows and self.client.windows_enabled:
             setall(self.batch_labels, self.values_from_info("batch_delay", "batch.delay"))
             setall(self.damage_labels, self.values_from_info("damage_out_latency", "damage.out_latency"))
             setall(self.quality_labels, self.all_values_from_info("quality", "encoding.quality"))
@@ -1017,8 +1052,8 @@ class SessionInfo(gtk.Window):
         return window_encoder_stats
 
 
-    def populate_graphs(self, *args):
-        self.client.send_info_request()
+    def populate_graphs(self, *_args):
+        self.client.send_info_request("network", "damage", "state", "batch")
         box = self.tab_box
         _, h = get_preferred_size(box)
         _, bh = get_preferred_size(self.tab_button_box)
@@ -1044,24 +1079,24 @@ class SessionInfo(gtk.Window):
                     if value==1:
                         return str(unit)
                     return "x%s%s" % (int(value), unit)
-            net_in_scale, net_in_data = values_to_diff_scaled_values(list(self.net_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
-            net_out_scale, net_out_data = values_to_diff_scaled_values(list(self.net_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            net_in_scale, net_in_data = values_to_diff_scaled_values(tuple(self.net_in_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+            net_out_scale, net_out_data = values_to_diff_scaled_values(tuple(self.net_out_bytecount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             if SHOW_RECV:
                 labels += ["recv %sB/s" % unit(net_in_scale), "sent %sB/s" % unit(net_out_scale)]
                 datasets += [net_in_data, net_out_data]
             else:
                 labels += ["recv %sB/s" % unit(net_in_scale)]
                 datasets += [net_in_data]
-        if SHOW_PIXEL_STATS and self.client.windows_enabled:
-            pixel_scale, in_pixels = values_to_scaled_values(list(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
+        if mixin_features.windows and SHOW_PIXEL_STATS and self.client.windows_enabled:
+            pixel_scale, in_pixels = values_to_scaled_values(tuple(self.pixel_in_data)[3:N_SAMPLES+4], min_scaled_value=100)
             datasets.append(in_pixels)
             labels.append("%s pixels/s" % unit(pixel_scale))
-        if SHOW_SOUND_STATS and self.sound_in_bitcount:
-            sound_in_scale, sound_in_data =  values_to_diff_scaled_values(list(self.sound_in_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+        if mixin_features.audio and SHOW_SOUND_STATS and self.sound_in_bitcount:
+            sound_in_scale, sound_in_data = values_to_diff_scaled_values(tuple(self.sound_in_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             datasets.append(sound_in_data)
             labels.append("Speaker %sb/s" % unit(sound_in_scale))
-        if SHOW_SOUND_STATS and self.sound_out_bitcount:
-            sound_out_scale, sound_out_data =  values_to_diff_scaled_values(list(self.sound_out_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
+        if mixin_features.audio and SHOW_SOUND_STATS and self.sound_out_bitcount:
+            sound_out_scale, sound_out_data = values_to_diff_scaled_values(tuple(self.sound_out_bitcount)[1:N_SAMPLES+3], scale_unit=1000, min_scaled_value=50)
             datasets.append(sound_out_data)
             labels.append("Mic %sb/s" % unit(sound_out_scale))
 
@@ -1105,7 +1140,7 @@ class SessionInfo(gtk.Window):
         self.latency_graph.set_size_request(W, H)
         self.latency_graph.set_from_pixmap(pixmap, None)
 
-        if SHOW_SOUND_STATS and self.client.sound_sink:
+        if mixin_features.audio and SHOW_SOUND_STATS and self.client.sound_sink:
             #sound queue graph:
             queue_values, queue_labels = norm_lists((
                                  (self.sound_out_queue_max, "Max"),
