@@ -26,7 +26,7 @@ metalog = Logger("metadata")
 traylog = Logger("client", "tray")
 
 
-from xpra.gtk_common.gobject_compat import import_glib
+from xpra.gtk_common.gobject_compat import import_glib, is_gtk3
 from xpra.platform.gui import get_vrefresh, get_double_click_time, get_double_click_distance, get_native_system_tray_classes
 from xpra.platform.features import SYSTEM_TRAY_SUPPORTED
 from xpra.platform.paths import get_icon_filename
@@ -47,6 +47,9 @@ PAINT_FAULT_TELL = envbool("XPRA_PAINT_FAULT_INJECTION_TELL", True)
 WM_CLASS_CLOSEEXIT = os.environ.get("XPRA_WM_CLASS_CLOSEEXIT", "Xephyr").split(",")
 TITLE_CLOSEEXIT = os.environ.get("XPRA_TITLE_CLOSEEXIT", "Xnest").split(",")
 
+OR_FORCE_GRAB = os.environ.get("XPRA_OR_FORCE_GRAB", "sun-awt-X11")
+
+
 SKIP_DUPLICATE_BUTTON_EVENTS = envbool("XPRA_SKIP_DUPLICATE_BUTTON_EVENTS", True)
 REVERSE_HORIZONTAL_SCROLLING = envbool("XPRA_REVERSE_HORIZONTAL_SCROLLING", OSX)
 
@@ -55,6 +58,7 @@ ICON_OVERLAY = envint("XPRA_ICON_OVERLAY", 50)
 ICON_SHRINKAGE = envint("XPRA_ICON_SHRINKAGE", 75)
 SAVE_WINDOW_ICONS = envbool("XPRA_SAVE_WINDOW_ICONS", False)
 SAVE_CURSORS = envbool("XPRA_SAVE_CURSORS", False)
+MODAL_WINDOWS = envbool("XPRA_MODAL_WINDOWS", False)
 
 
 DRAW_TYPES = {bytes : "bytes", str : "bytes", tuple : "arrays", list : "arrays"}
@@ -94,6 +98,7 @@ class WindowClient(StubClientMixin):
 
         self.server_input_devices = None
         self.server_precise_wheel = False
+        self.server_pointer_relative = False
         self.input_devices = "auto"
 
         self.overlay_image = None
@@ -107,6 +112,7 @@ class WindowClient(StubClientMixin):
 
         self.border = None
         self.window_close_action = "forward"
+        self.modal_windows = MODAL_WINDOWS
 
         self._pid_to_signalwatcher = {}
         self._signalwatcher_to_wids = {}
@@ -121,9 +127,8 @@ class WindowClient(StubClientMixin):
         self._window_with_grab = None
         self._suspended_at = 0
         self._button_state = {}
-        self._on_handshake = []
 
-    def init(self, opts):
+    def init(self, opts, extra_args=[]):
         if opts.system_tray and SYSTEM_TRAY_SUPPORTED:
             try:
                 from xpra.client import client_tray
@@ -150,6 +155,17 @@ class WindowClient(StubClientMixin):
             self.pixel_depth = 0
 
         self.windows_enabled = opts.windows
+        if self.windows_enabled:
+            if opts.window_close not in ("forward", "ignore", "disconnect", "shutdown", "auto"):
+                self.window_close_action = "forward"
+                log.warn("Warning: invalid 'window-close' option: '%s'", opts.window_close)
+                log.warn(" using '%s'", self.window_close_action)
+            else:
+                self.window_close_action = opts.window_close
+        self.modal_windows = self.windows_enabled and opts.modal_windows
+
+        if opts.border:
+            self.parse_border(opts.border, extra_args)
 
         #mouse wheel:
         mw = (opts.mousewheel or "").lower().replace("-", "")
@@ -167,26 +183,18 @@ class WindowClient(StubClientMixin):
                     self.wheel_map[btn+1] = btn
                     self.wheel_map[btn] = btn+1
 
-
-    def init_ui(self, opts, extra_args=[]):
-        if opts.border:
-            self.parse_border(opts.border, extra_args)
-        if opts.window_close not in ("forward", "ignore", "disconnect", "shutdown", "auto"):
-            self.window_close_action = "forward"
-            log.warn("Warning: invalid 'window-close' option: '%s'", opts.window_close)
-            log.warn(" using '%s'", self.window_close_action)
-        else:
-            self.window_close_action = opts.window_close
-
         if ICON_OVERLAY>0 and ICON_OVERLAY<=100:
             icon_filename = get_icon_filename("xpra")
-            try:
-                from PIL import Image   #@UnresolvedImport
-                self.overlay_image = Image.open(icon_filename)
-            except Exception as e:
-                log.warn("Warning: failed to load overlay icon '%s':", icon_filename)
-                log.warn(" %s", e)
-
+            if icon_filename:
+                try:
+                    #make sure Pillow's PNG image loader doesn't spam the output with debug messages:
+                    import logging
+                    logging.getLogger("PIL.PngImagePlugin").setLevel(logging.INFO)
+                    from PIL import Image   #@UnresolvedImport
+                    self.overlay_image = Image.open(icon_filename)
+                except Exception as e:
+                    log.error("Error: failed to load overlay icon '%s':", icon_filename, exc_info=True)
+                    log.error(" %s", e)
         self._draw_queue = Queue()
         self._draw_thread = make_thread(self._draw_thread_loop, "draw")
 
@@ -214,6 +222,13 @@ class WindowClient(StubClientMixin):
         log("WindowClient.cleanup() done")
 
 
+    def set_modal_windows(self, modal_windows):
+        self.modal_windows = modal_windows
+        #re-set flag on all the windows:
+        for w in self._id_to_window.values():
+            modal = w._metadata.boolget("modal", False)
+            w.set_modal(modal)
+
     def set_windows_cursor(self, client_windows, new_cursor):
         raise NotImplementedError()
 
@@ -256,6 +271,9 @@ class WindowClient(StubClientMixin):
             ):
             caps[x] = True
         updict(caps, "window", self.get_window_caps())
+        updict(caps, "encoding", {
+            "eos"                       : True,
+            })
         return caps
 
     def get_window_caps(self):
@@ -277,10 +295,13 @@ class WindowClient(StubClientMixin):
         self.default_cursor_data = c.listget("cursor.default", None)
         self.server_bell = c.boolget("bell")          #added in 0.5, default to True!
         self.bell_enabled = self.server_bell and self.client_supports_bell
-        if self.windows_enabled:
-            server_auto_refresh_delay = c.intget("auto_refresh_delay", 0)/1000.0
-            if server_auto_refresh_delay==0 and self.auto_refresh_delay>0:
-                log.warn("Warning: server does not support auto-refresh!")
+        if c.boolget("windows", True):
+            if self.windows_enabled:
+                server_auto_refresh_delay = c.intget("auto_refresh_delay", 0)/1000.0
+                if server_auto_refresh_delay==0 and self.auto_refresh_delay>0:
+                    log.warn("Warning: server does not support auto-refresh!")
+        else:
+            log.warn("Warning: window forwarding is not enabled on this server")
         self.server_window_signals = c.strlistget("window.signals")
         self.server_window_states = c.strlistget("window.states", ["iconified", "fullscreen", "above", "below", "sticky", "iconified", "maximized"])
         self.server_window_filters = c.boolget("window-filters")
@@ -288,6 +309,7 @@ class WindowClient(StubClientMixin):
         #input devices:
         self.server_input_devices = c.strget("input-devices")
         self.server_precise_wheel = c.boolget("wheel.precise", False)
+        self.server_pointer_relative = c.boolget("pointer.relative", False)
         return True
 
 
@@ -632,20 +654,22 @@ class WindowClient(StubClientMixin):
         metalog("process_new_common: %s, metadata=%s, OR=%s", packet[1:7], metadata, override_redirect)
         assert wid not in self._id_to_window, "we already have a window %s: %s" % (wid, self._id_to_window.get(wid))
         if w<1 or h<1:
-            log.error("window dimensions are wrong: %sx%s", w, h)
+            log.error("Error: window %i dimensions %ix%i are invalid", wid, w, h)
             w, h = 1, 1
-        x = self.sx(x)
-        y = self.sy(y)
-        bw, bh = w, h
+        #scaled dimensions of window:
+        wx = self.sx(x)
+        wy = self.sy(y)
         ww = max(1, self.sx(w))
         wh = max(1, self.sy(h))
+        #backing size, same as original (server-side):
+        bw, bh = w, h
         client_properties = {}
         if len(packet)>=8:
             client_properties = packet[7]
-        geomlog("process_new_common: wid=%i, OR=%s, geometry(%s)=%s", wid, override_redirect, packet[2:6], (x, y, ww, wh, bw, bh))
-        self.make_new_window(wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties)
+        geomlog("process_new_common: wid=%i, OR=%s, geometry(%s)=%s / %s", wid, override_redirect, packet[2:6], (wx, wy, ww, wh), (bw, bh))
+        self.make_new_window(wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties)
 
-    def make_new_window(self, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties):
+    def make_new_window(self, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties):
         client_window_classes = self.get_client_window_classes(ww, wh, metadata, override_redirect)
         group_leader_window = self.get_group_leader(wid, metadata, override_redirect)
         #workaround for "popup" OR windows without a transient-for (like: google chrome popups):
@@ -671,7 +695,7 @@ class WindowClient(StubClientMixin):
         log("make_new_window(..) client_window_classes=%s, group_leader_window=%s", client_window_classes, group_leader_window)
         for cwc in client_window_classes:
             try:
-                window = cwc(self, group_leader_window, watcher_pid, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, self.max_window_size, self.default_cursor_data, self.pixel_depth)
+                window = cwc(self, group_leader_window, watcher_pid, wid, wx, wy, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, self.max_window_size, self.default_cursor_data, self.pixel_depth)
                 break
             except:
                 log.warn("failed to instantiate %s", cwc, exc_info=True)
@@ -682,6 +706,13 @@ class WindowClient(StubClientMixin):
         self._id_to_window[wid] = window
         self._window_to_id[window] = wid
         window.show()
+        if override_redirect and OR_FORCE_GRAB:
+            wm_class = metadata.get("class-instance")
+            if wm_class and len(wm_class)==2:
+                c = wm_class[0]
+                if any(c.startswith(x) for x in OR_FORCE_GRAB):
+                    grablog.warn("forcing grab for OR window %i, wm class '%s' matches %s", wid, c, OR_FORCE_GRAB)
+                    self.window_grab(window)
         return window
 
     ######################################################################
@@ -714,7 +745,10 @@ class WindowClient(StubClientMixin):
                 getChildReaper().add_process(proc, "signal listener for remote process %s" % pid, command="xpra_signal_listener", ignore=True, forget=True, callback=watcher_terminated)
                 log("using watcher pid=%i for server pid=%i", proc.pid, pid)
                 self._pid_to_signalwatcher[pid] = proc
-                proc.stdout_io_watch = glib.io_add_watch(proc.stdout, glib.IO_IN, self.signal_watcher_event, proc, pid, wid)
+                if is_gtk3():
+                    proc.stdout_io_watch = glib.io_add_watch(proc.stdout, glib.PRIORITY_DEFAULT, glib.IO_IN, self.signal_watcher_event, proc, pid, wid)
+                else:
+                    proc.stdout_io_watch = glib.io_add_watch(proc.stdout, glib.IO_IN, self.signal_watcher_event, proc, pid, wid)
         if proc:
             self._signalwatcher_to_wids.setdefault(proc, []).append(wid)
             return proc.pid
@@ -1016,7 +1050,7 @@ class WindowClient(StubClientMixin):
                         if signalwatcher.poll() is None:
                             os.kill(signalwatcher.pid, signal.SIGKILL)
                     except:
-                        log("destroy_window(%i, %s) error getting rid of signal watcher %s", wid, window, signalwatcher, exc_info=True)
+                        log("destroy_window(%i, %s) error killing signal watcher %s", wid, window, signalwatcher, exc_info=True)
                     #now remove any pids that use this watcher:
                     for pid, w in tuple(self._pid_to_signalwatcher.items()):
                         if w==signalwatcher:
@@ -1031,6 +1065,14 @@ class WindowClient(StubClientMixin):
                 pass
         self._id_to_window = {}
         self._window_to_id = {}
+        #signal watchers should have been killed in destroy_window(),
+        #make sure we don't leave any behind:
+        for signalwatcher in tuple(self._signalwatcher_to_wids.keys()):
+            try:
+                if signalwatcher.poll() is None:
+                    os.kill(signalwatcher.pid, signal.SIGKILL)
+            except:
+                log("destroy_all_windows() error killing signal watcher %s", signalwatcher, exc_info=True)
 
 
     ######################################################################
@@ -1179,6 +1221,9 @@ class WindowClient(StubClientMixin):
     def _process_draw(self, packet):
         self._draw_queue.put(packet)
 
+    def _process_eos(self, packet):
+        self._draw_queue.put(packet)
+
     def send_damage_sequence(self, wid, packet_sequence, width, height, decode_time, message=""):
         packet = "damage-sequence", packet_sequence, wid, width, height, decode_time, message
         drawlog("sending ack: %s", packet)
@@ -1194,15 +1239,19 @@ class WindowClient(StubClientMixin):
                 sleep(0)
             except KeyboardInterrupt:
                 raise
-            except:
-                log.error("error processing draw packet", exc_info=True)
+            except Exception as e:
+                log.error("Error '%s' processing %s packet", e, packet[0], exc_info=True)
         log("draw thread ended")
 
     def _do_draw(self, packet):
         """ this runs from the draw thread above """
-        wid, x, y, width, height, coding, data, packet_sequence, rowstride = packet[1:10]
-        #rename old encoding aliases early:
+        wid = packet[1]
         window = self._id_to_window.get(wid)
+        if packet[0]==b"eos":
+            if window:
+                window.eos()
+            return
+        x, y, width, height, coding, data, packet_sequence, rowstride = packet[2:10]
         if not window:
             #window is gone
             def draw_cleanup():
@@ -1219,6 +1268,7 @@ class WindowClient(StubClientMixin):
                 self.send_damage_sequence(wid, packet_sequence, width, height, -1)
             self.idle_add(draw_cleanup)
             return
+        #rename old encoding aliases early:
         options = {}
         if len(packet)>10:
             options = packet[10]
@@ -1314,6 +1364,7 @@ class WindowClient(StubClientMixin):
             "lost-window":          self._process_lost_window,
             "window-icon":          self._process_window_icon,
             "draw":                 self._process_draw,
+            "eos":                  self._process_eos,
             "cursor":               self._process_cursor,
             "bell":                 self._process_bell,
             "pointer-position":     self._process_pointer_position,
