@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,9 +10,11 @@ import logging
 import weakref
 # This module is used by non-GUI programs and thus must not import gtk.
 
-LOG_PREFIX = os.environ.get("XPRA_LOG_PREFIX", "")
-LOG_FORMAT = os.environ.get("XPRA_LOG_FORMAT", "%(asctime)s %(message)s")
-NOPREFIX_FORMAT = "%(message)s"
+from xpra.os_util import bytestostr
+
+LOG_PREFIX = bytestostr(os.environ.get("XPRA_LOG_PREFIX", ""))
+LOG_FORMAT = bytestostr(os.environ.get("XPRA_LOG_FORMAT", "%(asctime)s %(message)s"))
+NOPREFIX_FORMAT = u"%(message)s"
 
 logging.basicConfig(format=LOG_FORMAT)
 logging.root.setLevel(logging.INFO)
@@ -40,6 +42,27 @@ def get_all_loggers():
                 a.add(v)
     return a
 
+
+class FullDebugContext(object):
+
+    def __enter__(self):
+        global debug_enabled_categories
+        self.debug_enabled_categories = debug_enabled_categories
+        debug_enabled_categories = set(debug_enabled_categories)
+        debug_enabled_categories.add("all")
+        self.enabled = []
+        for x in get_all_loggers():
+            if not x.is_debug_enabled():
+                self.enabled.append(x)
+                x.enable_debug()
+
+    def __exit__(self, *_args):
+        for x in self.enabled:
+            x.disable_debug()
+        global debug_enabled_categories
+        debug_enabled_categories = self.debug_enabled_categories
+
+
 debug_enabled_categories = set()
 debug_disabled_categories = set()
 def add_debug_category(*cat):
@@ -180,6 +203,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("frame"        , "Window frame"),
                 ("grab"         , "Window grabs (both keyboard and mouse)"),
                 ("dragndrop"    , "Window drag-n-drop events"),
+                ("filters"      , "Window filters"),
                 ])),
     ("Encoding", OrderedDict([
                 ("codec"        , "Codec loader and video helper"),
@@ -216,6 +240,8 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("nvfbc"        , "nfbc screen capture"),
                 ("x264"         , "libx264 encoder"),
                 ("x265"         , "libx265 encoder"),
+                ("xvid"         , "Xvid encoder"),
+                ("webp"         , "libwebp encoder and decoder"),
                 ("webcam"       , "webcam access"),
                 ])),
     ("Pointer", OrderedDict([
@@ -235,11 +261,16 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
     ("Network", OrderedDict([
                 #internal / network:
                 ("network"      , "All network code"),
+                ("bandwidth"    , "Bandwidth detection and management"),
+                ("ssh"          , "SSH connections"),
+                ("ssl"          , "SSL connections"),
                 ("http"         , "HTTP requests"),
+                ("rfb"          , "RFB Protocol"),
                 ("mmap"         , "mmap transfers"),
                 ("protocol"     , "Packet input and output (formatting, parsing, sending and receiving)"),
                 ("websocket"    , "WebSocket layer"),
                 ("named-pipe"   , "Named pipe"),
+                ("udp"          , "UDP"),
                 ("crypto"       , "Encryption"),
                 ("auth"         , "Authentication"),
                 ])),
@@ -288,7 +319,7 @@ for category, d in STRUCT_KNOWN_FILTERS.items():
 
 
 def isenvdebug(category):
-    return os.environ.get("XPRA_%s_DEBUG" % category.upper(), "0")=="1"
+    return os.environ.get("XPRA_%s_DEBUG" % category.upper().replace("-", "_"), "0")=="1"
 
 # A wrapper around 'logging' with some convenience stuff.  In particular:
 #    -- You initialize it with a list of categories
@@ -353,7 +384,7 @@ class Logger(object):
         if kwargs.get("exc_info") is True:
             ei = sys.exc_info()
             if ei!=(None, None, None):
-                kwargs["exc_info"] = sys.exc_info()
+                kwargs["exc_info"] = ei
         global global_logging_handler
         if LOG_PREFIX:
             msg = LOG_PREFIX+msg
