@@ -10,7 +10,6 @@
 
 import os
 from socket import error as socket_error
-import binascii
 from threading import Lock, Event
 
 
@@ -18,18 +17,18 @@ from xpra.log import Logger
 log = Logger("network", "protocol")
 cryptolog = Logger("network", "crypto")
 
-from xpra.os_util import PYTHON3, Queue, memoryview_to_bytes, strtobytes
+from xpra.os_util import PYTHON3, Queue, memoryview_to_bytes, strtobytes, bytestostr, hexstr
 from xpra.util import repr_ellipsized, csv, envint, envbool
 from xpra.make_thread import make_thread, start_thread
 from xpra.net.common import ConnectionClosedException          #@UndefinedVariable (pydev false positive)
 from xpra.net.bytestreams import ABORT
 from xpra.net import compression
 from xpra.net import packet_encoding
-from xpra.net.compression import get_compression_caps, decompress, sanity_checks as compression_sanity_checks,\
+from xpra.net.compression import decompress, sanity_checks as compression_sanity_checks,\
         InvalidCompressionException, Compressed, LevelCompressed, Compressible, LargeStructure
-from xpra.net.packet_encoding import get_packet_encoding_caps, decode, sanity_checks as packet_encoding_sanity_checks, InvalidPacketEncodingException
+from xpra.net.packet_encoding import decode, sanity_checks as packet_encoding_sanity_checks, InvalidPacketEncodingException
 from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER
-from xpra.net.crypto import get_crypto_caps, get_encryptor, get_decryptor, pad, INITIAL_PADDING
+from xpra.net.crypto import get_encryptor, get_decryptor, pad, INITIAL_PADDING
 
 
 #stupid python version breakage:
@@ -45,6 +44,7 @@ READ_BUFFER_SIZE = envint("XPRA_READ_BUFFER_SIZE", 65536)
 #merge header and packet if packet is smaller than:
 PACKET_JOIN_SIZE = envint("XPRA_PACKET_JOIN_SIZE", READ_BUFFER_SIZE)
 LARGE_PACKET_SIZE = envint("XPRA_LARGE_PACKET_SIZE", 4096)
+LOG_RAW_PACKET_SIZE = envbool("XPRA_LOG_RAW_PACKET_SIZE", False)
 #inline compressed data in packet if smaller than:
 INLINE_SIZE = envint("XPRA_INLINE_SIZE", 32768)
 FAKE_JITTER = envint("XPRA_FAKE_JITTER", 0)
@@ -53,37 +53,75 @@ SEND_INVALID_PACKET = envint("XPRA_SEND_INVALID_PACKET", 0)
 SEND_INVALID_PACKET_DATA = strtobytes(os.environ.get("XPRA_SEND_INVALID_PACKET_DATA", b"ZZinvalid-packetZZ"))
 
 
-def get_digests():
-    import hashlib
-    return ["hmac", "xor"] + ["hmac+%s" % x for x in list(reversed(sorted(hashlib.algorithms_available)))]
-
-def get_network_caps():
-    try:
-        from xpra.platform.features import MMAP_SUPPORTED
-    except:
-        MMAP_SUPPORTED = False
-    digests = get_digests()
-    #"hmac" is the legacy name, "xor" and "des" should not be used for salt:
-    salt_digests = [x for x in digests if x not in ("hmac", "xor", "des")]
-    caps = {
-                "digest"                : digests,
-                "salt-digest"           : salt_digests,
-                "compressors"           : compression.get_enabled_compressors(),
-                "encoders"              : packet_encoding.get_enabled_encoders(),
-                "mmap"                  : MMAP_SUPPORTED,
-               }
-    caps.update(get_crypto_caps())
-    caps.update(get_compression_caps())
-    caps.update(get_packet_encoding_caps())
-    return caps
-
 def sanity_checks():
     """ warns the user if important modules are missing """
     compression_sanity_checks()
     packet_encoding_sanity_checks()
 
 
+def exit_queue():
+    queue = Queue()
+    for _ in range(10):     #just 2 should be enough!
+        queue.put(None)
+    return queue
+
+def force_flush_queue(q):
+    try:
+        #discard all elements in the old queue and push the None marker:
+        try:
+            while q.qsize()>0:
+                q.read(False)
+        except:
+            pass
+        q.put_nowait(None)
+    except:
+        pass
+
+
+def verify_packet(packet):
+    """ look for None values which may have caused the packet to fail encoding """
+    if type(packet)!=list:
+        return False
+    assert len(packet)>0, "invalid packet: %s" % packet
+    tree = ["'%s' packet" % packet[0]]
+    return do_verify_packet(tree, packet)
+
+def do_verify_packet(tree, packet):
+    def err(msg):
+        log.error("%s in %s", msg, "->".join(tree))
+    def new_tree(append):
+        nt = tree[:]
+        nt.append(append)
+        return nt
+    if packet is None:
+        err("None value")
+        return False
+    r = True
+    if type(packet) in (list, tuple):
+        for i, x in enumerate(packet):
+            if not do_verify_packet(new_tree("[%s]" % i), x):
+                r = False
+    elif type(packet)==dict:
+        for k,v in packet.items():
+            if not do_verify_packet(new_tree("key for value='%s'" % str(v)), k):
+                r = False
+            if not do_verify_packet(new_tree("value for key='%s'" % str(k)), v):
+                r = False
+    elif type(packet) in (int, bool, str, bytes, unicode):
+        pass
+    else:
+        err("unsupported type: %s" % type(packet))
+        r = False
+    return r
+
+
 class Protocol(object):
+    """
+        This class handles sending and receiving packets,
+        it will encode and compress them before sending,
+        and decompress and decode when receiving.
+    """
+
     CONNECTION_LOST = "connection-lost"
     GIBBERISH = "gibberish"
     INVALID = "invalid"
@@ -96,6 +134,7 @@ class Protocol(object):
         assert conn is not None
         self.timeout_add = scheduler.timeout_add
         self.idle_add = scheduler.idle_add
+        self.source_remove = scheduler.source_remove
         self._conn = conn
         if FAKE_JITTER>0:
             from xpra.net.fake_jitter import FakeJitter
@@ -105,6 +144,7 @@ class Protocol(object):
             self._process_packet_cb = process_packet_cb
         self._write_queue = Queue(1)
         self._read_queue = Queue(20)
+        self._process_read = self.read_queue_put
         self._read_queue_put = self.read_queue_put
         # Invariant: if .source is None, then _source_has_more == False
         self._get_packet_cb = get_packet_cb
@@ -118,7 +158,7 @@ class Protocol(object):
         #initial value which may get increased by client/server after handshake:
         self.max_packet_size = 256*1024
         self.abs_max_packet_size = 256*1024*1024
-        self.large_packets = ["hello", "window-metadata", "sound-data"]
+        self.large_packets = [b"hello", b"window-metadata", b"sound-data", b"notify_show"]
         self.send_aliases = {}
         self.receive_aliases = {}
         self._log_stats = None          #None here means auto-detect
@@ -162,14 +202,16 @@ class Protocol(object):
         self.enable_compressor(self.compressor)
         self.enable_encoder(self.encoder)
 
+
     def wait_for_io_threads_exit(self, timeout=None):
-        for t in (self._read_thread, self._write_thread):
-            if t and t.isAlive():
+        io_threads = [x for x in (self._read_thread, self._write_thread) if x is not None]
+        for t in io_threads:
+            if t.isAlive():
                 t.join(timeout)
         exited = True
         cinfo = self._conn or "cleared connection"
-        for t in (self._read_thread, self._write_thread):
-            if t and t.isAlive():
+        for t in io_threads:
+            if t.isAlive():
                 log.warn("Warning: %s thread of %s is still alive (timeout=%s)", t.name, cinfo, timeout)
                 exited = False
         return exited
@@ -179,20 +221,20 @@ class Protocol(object):
 
 
     def set_cipher_in(self, ciphername, iv, password, key_salt, iterations, padding):
-        if self.cipher_in_name!=ciphername:
-            cryptolog.info("receiving data using %s encryption", ciphername)
-            self.cipher_in_name = ciphername
         cryptolog("set_cipher_in%s", (ciphername, iv, password, key_salt, iterations))
         self.cipher_in, self.cipher_in_block_size = get_decryptor(ciphername, iv, password, key_salt, iterations)
         self.cipher_in_padding = padding
+        if self.cipher_in_name!=ciphername:
+            cryptolog.info("receiving data using %s encryption", ciphername)
+            self.cipher_in_name = ciphername
 
     def set_cipher_out(self, ciphername, iv, password, key_salt, iterations, padding):
-        if self.cipher_out_name!=ciphername:
-            cryptolog.info("sending data using %s encryption", ciphername)
-            self.cipher_out_name = ciphername
         cryptolog("set_cipher_out%s", (ciphername, iv, password, key_salt, iterations, padding))
         self.cipher_out, self.cipher_out_block_size = get_encryptor(ciphername, iv, password, key_salt, iterations)
         self.cipher_out_padding = padding
+        if self.cipher_out_name!=ciphername:
+            cryptolog.info("sending data using %s encryption", ciphername)
+            self.cipher_out_name = ciphername
 
 
     def __repr__(self):
@@ -201,10 +243,13 @@ class Protocol(object):
     def get_threads(self):
         return  [x for x in [self._write_thread, self._read_thread, self._read_parser_thread, self._write_format_thread] if x is not None]
 
+    def accept(self):
+        pass
+
 
     def get_info(self, alias_info=True):
         info = {
-            "large_packets"         : self.large_packets,
+            "large_packets"         : tuple(bytestostr(x) for x in self.large_packets),
             "compression_level"     : self.compression_level,
             "max_packet_size"       : self.max_packet_size,
             "aliases"               : USE_ALIASES,
@@ -248,7 +293,8 @@ class Protocol(object):
                 info.update(self._conn.get_info())
             except:
                 log.error("error collecting connection information on %s", self._conn, exc_info=True)
-        info["has_more"] = self._source_has_more.is_set()
+        shm = self._source_has_more
+        info["has_more"] = shm and shm.is_set()
         for t in (self._write_thread, self._read_thread, self._read_parser_thread, self._write_format_thread):
             if t:
                 info.setdefault("thread", {})[t.name] = t.is_alive()
@@ -263,6 +309,10 @@ class Protocol(object):
         if SEND_INVALID_PACKET:
             self.timeout_add(SEND_INVALID_PACKET*1000, self.raw_write, SEND_INVALID_PACKET_DATA)
 
+
+    def send_disconnect(self, reasons, done_callback=None):
+        self.flush_then_close(["disconnect"]+list(reasons), done_callback=done_callback)
+
     def send_now(self, packet):
         if self._closed:
             log("send_now(%s ...) connection is closed already, not sending", packet[0])
@@ -280,7 +330,10 @@ class Protocol(object):
         self.source_has_more()
 
     def source_has_more(self):
-        self._source_has_more.set()
+        shm = self._source_has_more
+        if not shm or self._closed:
+            return
+        shm.set()
         #start the format thread:
         if not self._write_format_thread and not self._closed:
             self._write_format_thread = make_thread(self._write_format_thread_loop, "format", daemon=True)
@@ -296,42 +349,34 @@ class Protocol(object):
                 gpc = self._get_packet_cb
                 if self._closed or not gpc:
                     return
-                self._source_has_more.clear()
                 self._add_packet_to_queue(*gpc())
         except Exception as e:
             if self._closed:
                 return
             self._internal_error("error in network packet write/format", e, exc_info=True)
 
-    def _add_packet_to_queue(self, packet, start_send_cb=None, end_send_cb=None, has_more=False):
-        if has_more:
-            self._source_has_more.set()
+    def _add_packet_to_queue(self, packet, start_send_cb=None, end_send_cb=None, fail_cb=None, synchronous=True, has_more=False, wait_for_more=False):
+        if not has_more:
+            self._source_has_more.clear()
         if packet is None:
             return
-        log("add_packet_to_queue(%s ...)", packet[0])
+        log("add_packet_to_queue(%s ... %s, %s, %s)", packet[0], synchronous, has_more, wait_for_more)
         chunks = self.encode(packet)
         with self._write_lock:
             if self._closed:
                 return
             try:
-                self._add_chunks_to_queue(chunks, start_send_cb, end_send_cb)
+                self._add_chunks_to_queue(chunks, start_send_cb, end_send_cb, fail_cb, synchronous, has_more or wait_for_more)
             except:
                 log.error("Error: failed to queue '%s' packet", packet[0])
-                log("add_chunks_to_queue%s", (chunks, start_send_cb, end_send_cb), exc_info=True)
+                log("add_chunks_to_queue%s", (chunks, start_send_cb, end_send_cb, fail_cb), exc_info=True)
                 raise
 
-    def _add_chunks_to_queue(self, chunks, start_send_cb=None, end_send_cb=None):
+    def _add_chunks_to_queue(self, chunks, start_send_cb=None, end_send_cb=None, fail_cb=None, synchronous=True, more=False):
         """ the write_lock must be held when calling this function """
         counter = 0
         items = []
         for proto_flags,index,level,data in chunks:
-            scb, ecb = None, None
-            #fire the start_send_callback just before the first packet is processed:
-            if counter==0:
-                scb = start_send_cb
-            #fire the end_send callback when the last packet (index==0) makes it out:
-            if index==0:
-                ecb = end_send_cb
             payload_size = len(data)
             actual_size = payload_size
             if self.cipher_out:
@@ -352,55 +397,27 @@ class Protocol(object):
                 assert not self.cipher_out
                 #for plain/text packets (ie: gibberish response)
                 log("sending %s bytes without header", payload_size)
-                items.append((data, scb, ecb))
+                items.append(data)
             elif actual_size<PACKET_JOIN_SIZE:
-                if type(data) not in JOIN_TYPES:
+                if not isinstance(data, JOIN_TYPES):
                     data = memoryview_to_bytes(data)
                 header_and_data = pack_header(proto_flags, level, index, payload_size) + data
-                items.append((header_and_data, scb, ecb))
+                items.append(header_and_data)
             else:
                 header = pack_header(proto_flags, level, index, payload_size)
-                items.append((header, scb, None))
-                items.append((data, None, ecb))
+                items.append(header)
+                items.append(data)
             counter += 1
-        if self._write_thread is None:
-            self.start_write_thread()
-        self._write_queue.put(items)
-        self.output_packetcount += 1
+        self.raw_write(items, start_send_cb, end_send_cb, fail_cb, synchronous, more)
 
     def start_write_thread(self):
         self._write_thread = start_thread(self._write_thread_loop, "write", daemon=True)
 
-    def raw_write(self, contents, start_cb=None, end_cb=None):
+    def raw_write(self, items, start_cb=None, end_cb=None, fail_cb=None, synchronous=True, more=False):
         """ Warning: this bypasses the compression and packet encoder! """
         if self._write_thread is None:
             self.start_write_thread()
-        self._write_queue.put(((contents, start_cb, end_cb), ))
-
-    def verify_packet(self, packet):
-        """ look for None values which may have caused the packet to fail encoding """
-        if type(packet)!=list:
-            return
-        assert len(packet)>0, "invalid packet: %s" % packet
-        tree = ["'%s' packet" % packet[0]]
-        self.do_verify_packet(tree, packet)
-
-    def do_verify_packet(self, tree, packet):
-        def err(msg):
-            log.error("%s in %s", msg, "->".join(tree))
-        def new_tree(append):
-            nt = tree[:]
-            nt.append(append)
-            return nt
-        if packet is None:
-            return err("None value")
-        if type(packet)==list:
-            for i, x in enumerate(packet):
-                self.do_verify_packet(new_tree("[%s]" % i), x)
-        elif type(packet)==dict:
-            for k,v in packet.items():
-                self.do_verify_packet(new_tree("key for value='%s'" % str(v)), k)
-                self.do_verify_packet(new_tree("value for key='%s'" % str(k)), v)
+        self._write_queue.put((items, start_cb, end_cb, fail_cb, synchronous, more))
 
 
     def enable_default_encoder(self):
@@ -530,7 +547,7 @@ class Protocol(object):
                 #replace this item with an empty string placeholder:
                 packet[i] = ''
             elif ti not in (str, bytes):
-                log.warn("unexpected data type %s in %s packet: %s", ti, packet[0], repr_ellipsized(item))
+                log.warn("Warning: unexpected data type %s in '%s' packet: %s", ti, packet[0], repr_ellipsized(item))
         #now the main packet (or what is left of it):
         packet_type = packet[0]
         self.output_stats[packet_type] = self.output_stats.get(packet_type, 0)+1
@@ -545,9 +562,9 @@ class Protocol(object):
             log.error("failed to encode packet: %s", packet, exc_info=True)
             #make the error a bit nicer to parse: undo aliases:
             packet[0] = packet_type
-            self.verify_packet(packet)
+            verify_packet(packet)
             raise
-        if len(main_packet)>size_check and packet_in[0] not in self.large_packets:
+        if len(main_packet)>size_check and strtobytes(packet_in[0]) not in self.large_packets:
             log.warn("found large packet (%s bytes): %s, argument types:%s, sizes: %s, packet head=%s",
                      len(main_packet), packet_in[0], [type(x) for x in packet[1:]], [len(str(x)) for x in packet[1:]], repr_ellipsized(packet))
         #compress, but don't bother for small packets:
@@ -563,13 +580,14 @@ class Protocol(object):
         assert level>=0 and level<=10, "invalid compression level: %s (must be between 0 and 10" % level
         self.compression_level = level
 
+
     def _io_thread_loop(self, name, callback):
         try:
             log("io_thread_loop(%s, %s) loop starting", name, callback)
             while not self._closed and callback():
                 pass
             log("io_thread_loop(%s, %s) loop ended, closed=%s", name, callback, self._closed)
-        except ConnectionClosedException as e:
+        except ConnectionClosedException:
             log("%s closed", self._conn, exc_info=True)
             if not self._closed:
                 #ConnectionClosedException means the warning has been logged already
@@ -583,6 +601,7 @@ class Protocol(object):
                 log.error("Error: %s on %s failed: %s", name, self._conn, type(e), exc_info=True)
                 self.close()
 
+
     def _write_thread_loop(self):
         self._io_thread_loop("write", self._write)
     def _write(self):
@@ -592,16 +611,33 @@ class Protocol(object):
             log("write thread: empty marker, exiting")
             self.close()
             return False
-        for buf, start_cb, end_cb in items:
-            con = self._conn
-            if not con:
-                return False
-            if start_cb:
-                try:
-                    start_cb(con.output_bytecount)
-                except:
-                    if not self._closed:
-                        log.error("Error on write start callback %s", start_cb, exc_info=True)
+        return self.write_items(*items)
+
+    def write_items(self, buf_data, start_cb=None, end_cb=None, fail_cb=None, synchronous=True, more=False):
+        conn = self._conn
+        if not conn:
+            return False
+        conn.set_nodelay(not more)
+        if start_cb:
+            try:
+                start_cb(conn.output_bytecount)
+            except:
+                if not self._closed:
+                    log.error("Error on write start callback %s", start_cb, exc_info=True)
+        self.write_buffers(buf_data, fail_cb, synchronous)
+        if end_cb:
+            try:
+                end_cb(self._conn.output_bytecount)
+            except:
+                if not self._closed:
+                    log.error("Error on write end callback %s", end_cb, exc_info=True)
+        return True
+
+    def write_buffers(self, buf_data, _fail_cb, _synchronous):
+        con = self._conn
+        if not con:
+            return 0
+        for buf in buf_data:
             while buf and not self._closed:
                 written = con.write(buf)
                 #example test code, for sending small chunks very slowly:
@@ -611,13 +647,8 @@ class Protocol(object):
                 if written:
                     buf = buf[written:]
                     self.output_raw_packetcount += 1
-            if end_cb:
-                try:
-                    end_cb(self._conn.output_bytecount)
-                except:
-                    if not self._closed:
-                        log.error("Error on write end callback %s", end_cb, exc_info=True)
-        return True
+        self.output_packetcount += 1
+
 
     def _read_thread_loop(self):
         self._io_thread_loop("read", self._read)
@@ -625,7 +656,7 @@ class Protocol(object):
         buf = self._conn.read(READ_BUFFER_SIZE)
         #log("read thread: got data of size %s: %s", len(buf), repr_ellipsized(buf))
         #add to the read queue (or whatever takes its place - see steal_connection)
-        self._read_queue_put(buf)
+        self._process_read(buf)
         if not buf:
             log("read thread: eof")
             #give time to the parse thread to call close itself
@@ -645,6 +676,7 @@ class Protocol(object):
         log.error("Error: %s", message, exc_info=ei)
         if exc:
             log.error(" %s", exc, exc_info=exc_info)
+            exc = None
         self.idle_add(self._connection_lost, message)
 
     def _connection_lost(self, message="", exc_info=False):
@@ -670,13 +702,16 @@ class Protocol(object):
     def _invalid_header(self, data, msg=""):
         self.invalid_header(self, data, msg)
 
-    def invalid_header(self, proto, data, msg="invalid packet header"):
-        err = "%s: '%s'" % (msg, binascii.hexlify(data[:8]))
+    def invalid_header(self, _proto, data, msg="invalid packet header"):
+        err = "%s: '%s'" % (msg, hexstr(data[:8]))
         if len(data)>1:
             err += " read buffer=%s (%i bytes)" % (repr_ellipsized(data), len(data))
         self.gibberish(err, data)
 
 
+    def process_read(self, data):
+        self._read_queue_put(data)
+
     def read_queue_put(self, data):
         #start the parse thread if needed:
         if not self._read_parser_thread and not self._closed:
@@ -684,13 +719,15 @@ class Protocol(object):
                 log("empty marker in read queue, exiting")
                 self.idle_add(self.close)
                 return
-            self._read_parser_thread = make_thread(self._read_parse_thread_loop, "parse", daemon=True)
-            self._read_parser_thread.start()
+            self.start_read_parser_thread()
         self._read_queue.put(data)
         #from now on, take shortcut:
         if self._read_queue_put==self.read_queue_put:
             self._read_queue_put = self._read_queue.put
 
+    def start_read_parser_thread(self):
+        self._read_parser_thread = start_thread(self._read_parse_thread_loop, "parse", daemon=True)
+
     def _read_parse_thread_loop(self):
         log("read_parse_thread_loop starting")
         try:
@@ -713,6 +750,7 @@ class Protocol(object):
             from the UI thread will need to use a callback (usually via 'idle_add')
         """
         read_buffer = None
+        packet_size = 0
         payload_size = -1
         padding_size = 0
         packet_index = 0
@@ -741,7 +779,7 @@ class Protocol(object):
                         return
                     if bl<8:
                         break   #packet still too small
-                    #packet format: struct.pack('cBBBL', ...) - 8 bytes
+                    #packet format: struct.pack(b'cBBBL', ...) - 8 bytes
                     _, protocol_flags, compression_level, packet_index, data_size = unpack_header(read_buffer[:8])
 
                     #sanity check size (will often fail if not an xpra client):
@@ -789,6 +827,7 @@ class Protocol(object):
                 else:
                     raw_string = read_buffer[:payload_size]
                     read_buffer = read_buffer[payload_size:]
+                packet_size += 8+payload_size
                 #decrypt if needed:
                 data = raw_string
                 if self.cipher_in and protocol_flags & FLAGS_CIPHER:
@@ -797,9 +836,9 @@ class Protocol(object):
                     if padding_size > 0:
                         def debug_str(s):
                             try:
-                                return binascii.hexlify(bytearray(s))
+                                return hexstr(bytearray(s))
                             except:
-                                return csv(list(str(s)))
+                                return csv(tuple(s))
                         # pad byte value is number of padding bytes added
                         padtext = pad(self.cipher_in_padding, padding_size)
                         if data.endswith(padtext):
@@ -810,7 +849,8 @@ class Protocol(object):
                             cryptolog(" data does not end with %s padding bytes %s", self.cipher_in_padding, debug_str(padtext))
                             cryptolog(" but with %s (%s)", debug_str(actual_padding), type(data))
                             cryptolog(" decrypted data: %s", debug_str(data[:128]))
-                            return self._internal_error("%s encryption padding error - wrong key?" % self.cipher_in_name)
+                            self._internal_error("%s encryption padding error - wrong key?" % self.cipher_in_name)
+                            return
                         data = data[:-padding_size]
                 #uncompress if needed:
                 if compression_level>0:
@@ -829,6 +869,7 @@ class Protocol(object):
                             #only include the exception text when not using encryption
                             #as this may leak crypto information:
                             msg += " %s" % e
+                        del e
                         return self.gibberish(msg, data)
 
                 if self.cipher_in and not (protocol_flags & FLAGS_CIPHER):
@@ -858,7 +899,7 @@ class Protocol(object):
                     log.error(" %s", e)
                     if self._closed:
                         return
-                    log("failed to parse %s packet: %s", etype, binascii.hexlify(data[:128]))
+                    log("failed to parse %s packet: %s", etype, hexstr(data[:128]))
                     log(" %s", e)
                     log(" data: %s", repr_ellipsized(data))
                     log(" packet index=%i, packet size=%i, buffer size=%s", packet_index, payload_size, bl)
@@ -881,9 +922,12 @@ class Protocol(object):
                     packet_type = self.receive_aliases.get(packet_type)
                     packet[0] = packet_type
                 self.input_stats[packet_type] = self.output_stats.get(packet_type, 0)+1
+                if LOG_RAW_PACKET_SIZE:
+                    log("%s: %i bytes", packet_type, packet_size)
+                packet_size = 0
 
                 self.input_packetcount += 1
-                log("processing packet %s", packet_type)
+                log("processing packet %s", bytestostr(packet_type))
                 self._process_packet_cb(self, packet)
                 packet = None
 
@@ -911,7 +955,10 @@ class Protocol(object):
                 #write queue still has stuff in it..
                 if timeout<=0:
                     log("flush_then_close: queue still busy, closing without sending the last packet")
-                    self._write_lock.release()
+                    try:
+                        self._write_lock.release()
+                    except:
+                        pass
                     self.close()
                     done()
                 else:
@@ -935,13 +982,13 @@ class Protocol(object):
                         close_and_release()
                         return False
                     return not self._closed     #run until we manage to close (here or via the timeout)
-                def packet_queued(*args):
+                def packet_queued(*_args):
                     #if we're here, we have the lock and the packet is in the write queue
                     log("flush_then_close: packet_queued() closed=%s", self._closed)
                     if wait_for_packet_sent():
                         #check again every 100ms
                         self.timeout_add(100, wait_for_packet_sent)
-                self._add_chunks_to_queue(chunks, start_send_cb=None, end_send_cb=packet_queued)
+                self._add_chunks_to_queue(chunks, start_send_cb=None, end_send_cb=packet_queued, synchronous=False, more=False)
                 #just in case wait_for_packet_sent never fires:
                 self.timeout_add(5*1000, close_and_release)
 
@@ -976,21 +1023,21 @@ class Protocol(object):
         self.idle_add(self._process_packet_cb, self, [Protocol.CONNECTION_LOST])
         c = self._conn
         if c:
+            self._conn = None
             try:
                 log("Protocol.close() calling %s", c.close)
                 c.close()
-                if self._log_stats is None and self._conn.input_bytecount==0 and self._conn.output_bytecount==0:
+                if self._log_stats is None and c.input_bytecount==0 and c.output_bytecount==0:
                     #no data sent or received, skip logging of stats:
                     self._log_stats = False
                 if self._log_stats:
                     from xpra.simple_stats import std_unit, std_unit_dec
                     log.info("connection closed after %s packets received (%s bytes) and %s packets sent (%s bytes)",
-                         std_unit(self.input_packetcount), std_unit_dec(self._conn.input_bytecount),
-                         std_unit(self.output_packetcount), std_unit_dec(self._conn.output_bytecount)
+                         std_unit(self.input_packetcount), std_unit_dec(c.input_bytecount),
+                         std_unit(self.output_packetcount), std_unit_dec(c.output_bytecount)
                          )
             except:
-                log.error("error closing %s", self._conn, exc_info=True)
-            self._conn = None
+                log.error("error closing %s", c, exc_info=True)
         self.terminate_queue_threads()
         self.idle_add(self.clean)
         log("Protocol.close() done")
@@ -1024,6 +1071,16 @@ class Protocol(object):
         self._read_parser_thread = None
         self._write_format_thread = None
         self._process_packet_cb = None
+        self._process_read = None
+        self._read_queue_put = None
+        self._compress = None
+        self._write_lock = None
+        self._source_has_more = None
+        self._conn = None       #should be redundant
+        def noop():
+            pass
+        self.source_has_more = noop
+
 
     def terminate_queue_threads(self):
         log("terminate_queue_threads()")
@@ -1031,32 +1088,13 @@ class Protocol(object):
         self._get_packet_cb = None
         self._source_has_more.set()
         #make all the queue based threads exit by adding the empty marker:
-        exit_queue = Queue()
-        for _ in range(10):     #just 2 should be enough!
-            exit_queue.put(None)
-        try:
-            owq = self._write_queue
-            self._write_queue = exit_queue
-            #discard all elements in the old queue and push the None marker:
-            try:
-                while owq.qsize()>0:
-                    owq.read(False)
-            except:
-                pass
-            owq.put_nowait(None)
-        except:
-            pass
-        try:
-            orq = self._read_queue
-            self._read_queue = exit_queue
-            #discard all elements in the old queue and push the None marker:
-            try:
-                while orq.qsize()>0:
-                    orq.read(False)
-            except:
-                pass
-            orq.put_nowait(None)
-        except:
-            pass
+        #write queue:
+        owq = self._write_queue
+        self._write_queue = exit_queue()
+        force_flush_queue(owq)
+        #read queue:
+        orq = self._read_queue
+        self._read_queue = exit_queue()
+        force_flush_queue(orq)
         #just in case the read thread is waiting again:
         self._source_has_more.set()
