@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,6 +11,7 @@ import sys
 import signal
 import uuid
 import time
+import binascii
 
 #hide some ugly python3 compat:
 try:
@@ -62,12 +63,17 @@ POSIX = os.name=="posix"
 PYTHON2 = sys.version_info[0]==2
 PYTHON3 = sys.version_info[0]==3
 
+import struct
+BITS = struct.calcsize(b"P")*8
+
 
 if PYTHON2:
     def strtobytes(x):
         return str(x)
     def bytestostr(x):
         return str(x)
+    def hexstr(v):
+        return binascii.hexlify(str(v))
 else:
     def strtobytes(x):
         if type(x)==bytes:
@@ -77,6 +83,17 @@ else:
         if type(x)==bytes:
             return x.decode("latin1")
         return str(x)
+    def hexstr(v):
+        return bytestostr(binascii.hexlify(strtobytes(v)))
+
+
+util_logger = None
+def get_util_logger():
+    global util_logger
+    if not util_logger:
+        from xpra.log import Logger
+        util_logger = Logger("util")
+    return util_logger
 
 def memoryview_to_bytes(v):
     if type(v)==bytes:
@@ -87,7 +104,7 @@ def memoryview_to_bytes(v):
         return bytes(v)
     if isinstance(v, bytearray):
         return bytes(v)
-    return v
+    return strtobytes(v)
 
 
 def setsid():
@@ -150,14 +167,23 @@ def get_group_id(group):
 
 def platform_release(release):
     if OSX:
+        SYSTEMVERSION_PLIST = "/System/Library/CoreServices/SystemVersion.plist"
         try:
             import plistlib
-            pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+            if PYTHON2:
+                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+            else:
+                with open(SYSTEMVERSION_PLIST, "rb") as f:
+                    pl = plistlib.load(f)           #@UndefinedVariable
             return pl['ProductUserVisibleVersion']
-        except:
-            pass
+        except Exception as e:
+            get_util_logger().debug("platform_release(%s)", release, exc_info=True)
+            get_util_logger().warn("Warning: failed to get release information")
+            get_util_logger().warn(" from '%s':", SYSTEMVERSION_PLIST)
+            get_util_logger().warn(" %s", e)
     return release
 
+
 def platform_name(sys_platform, release):
     if not sys_platform:
         return "unknown"
@@ -182,6 +208,10 @@ def platform_name(sys_platform, release):
     return rel(sys_platform)
 
 
+def get_rand_chars(l=16, chars=b"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"):
+    import random
+    return b"".join(chars[random.randint(0, len(chars)-1)] for _ in range(l))
+
 def get_hex_uuid():
     return uuid.uuid4().hex
 
@@ -207,6 +237,9 @@ def get_user_uuid():
         Try to generate a uuid string which is unique to this user.
         (relies on get_machine_id to uniquely identify a machine)
     """
+    uuid = os.environ.get("XPRA_USER_UUID")
+    if uuid:
+        return uuid
     import hashlib
     u = hashlib.sha1()
     def uupdate(ustr):
@@ -229,6 +262,20 @@ try:
 except Exception as e:
     pass
 
+
+def is_X11():
+    if PYTHON2:
+        return True
+    if OSX or WIN32:
+        return False
+    from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
+    x11 = is_X11_Display()
+    return x11
+
+def is_Wayland():
+    return os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+
+
 def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
     if not POSIX:
         return False
@@ -239,7 +286,7 @@ def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
         pass
     try:
         v = load_binary_file(os_file)
-        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith("NAME="))
+        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith(b"NAME="))
     except:
         return False
 
@@ -327,7 +374,9 @@ def load_binary_file(filename):
     try:
         with open(filename, "rb") as f:
             return f.read()
-    except:
+    except Exception as e:
+        get_util_logger().warn("Warning: failed to load '%s':", filename)
+        get_util_logger().warn(" %s", e)
         return None
 
 #here so we can override it when needed
@@ -389,6 +438,12 @@ def close_all_fds(exceptions=[]):
             # by listdir() is already closed.
             pass
 
+def use_tty():
+    from xpra.util import envbool
+    NOTTY = envbool("XPRA_NOTTY", False)
+    stdin = sys.stdin
+    return not os.environ.get("MSYSCON") and not NOTTY and stdin and stdin.isatty()
+
 
 def shellsub(s, subs={}):
     """ shell style string substitution using the dictionary given """
@@ -397,6 +452,40 @@ def shellsub(s, subs={}):
         s = s.replace("${%s}" % var, str(value))
     return s
 
+
+def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
+    def expanduser(s):
+        if actual_username and s.startswith("~/"):
+            #replace "~/" with "~$actual_username/"
+            return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
+        return os.path.expanduser(s)
+    from collections import OrderedDict
+    d = OrderedDict(subs)
+    d.update({
+        "PID"   : os.getpid(),
+        "HOME"  : expanduser("~/"),
+        })
+    if os.name=="posix":
+        d.update({
+            "UID"   : uid or os.geteuid(),
+            "GID"   : gid or os.getegid(),
+            })
+        if not OSX:
+            from xpra.platform.xposix.paths import get_runtime_dir
+            d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
+    if actual_username:
+        d.update({
+            "USERNAME"  : actual_username,
+            "USER"      : actual_username,
+            })
+    #first, expand the substitutions themselves,
+    #as they may contain references to other variables:
+    ssub = OrderedDict()
+    for k,v in d.items():
+        ssub[k] = expanduser(shellsub(str(v), d))
+    return os.path.expandvars(expanduser(shellsub(expanduser(s), ssub)))
+
+
 def path_permission_info(filename, ftype=None):
     if not POSIX:
         return []
@@ -425,7 +514,7 @@ def path_permission_info(filename, ftype=None):
 #ideally we would redirect to a buffer so we could still capture and show these messages in debug out
 class HideStdErr(object):
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *_args):
         self.savedstderr = None
 
     def __enter__(self):
@@ -439,20 +528,20 @@ class HideStdErr(object):
         os.close(devnull)
         sys.stderr = os.fdopen(self.savedstderr, 'w')
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *_args):
         if self.savedstderr is not None:
             os.dup2(self.savedstderr, 2)
 
 class HideSysArgv(object):
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *_args):
         self.savedsysargv = None
 
     def __enter__(self):
         self.savedsysargv = sys.argv
         sys.argv = sys.argv[:1]
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *_args):
         if self.savedsysargv is not None:
             sys.argv = self.savedsysargv
 
@@ -463,7 +552,7 @@ class OSEnvContext(object):
         self.env = os.environ.copy()
     def __enter__(self):
         pass
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *_args):
         os.environ.clear()
         os.environ.update(self.env)
     def __repr__(self):
@@ -474,14 +563,14 @@ class FDChangeCaptureContext(object):
 
     def __enter__(self):
         self.enter_fds = get_all_fds()
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *_args):
         self.exit_fds = get_all_fds()
     def __repr__(self):
         return "FDChangeCaptureContext"
     def get_new_fds(self):
-        return sorted(list(set(self.exit_fds)-set(self.enter_fds)))
+        return sorted(tuple(set(self.exit_fds)-set(self.enter_fds)))
     def get_lost_fds(self):
-        return sorted(list(set(self.enter_fds)-set(self.exit_fds)))
+        return sorted(tuple(set(self.enter_fds)-set(self.exit_fds)))
 
 class DummyContextManager(object):
 
@@ -493,6 +582,33 @@ class DummyContextManager(object):
         return "DummyContextManager"
 
 
+#workaround incompatibility between paramiko and gssapi:
+class nomodule_context(object):
+
+    def __init__(self, module_name):
+        self.module_name = module_name
+        self.saved_module = sys.modules.get(module_name)
+        sys.modules[module_name] = None
+    def __enter__(self):
+        pass
+    def __exit__(self, *_args):
+        if sys.modules[self.module_name] == None:
+            sys.modules[self.module_name] = self.saved_module
+    def __repr__(self):
+        return "nomodule_context(%s)" % self.module_name
+
+class umask_context(object):
+
+    def __init__(self, umask):
+        self.umask = umask        
+    def __enter__(self):
+        self.orig_umask = os.umask(self.umask)
+    def __exit__(self, *_args):
+        os.umask(self.orig_umask)
+    def __repr__(self):
+        return "umask_context(%s)" % self.umask
+
+
 def disable_stdout_buffering():
     import gc
     # Appending to gc.garbage is a way to stop an object from being
@@ -508,9 +624,7 @@ def setbinarymode(fd):
             import msvcrt
             msvcrt.setmode(fd, os.O_BINARY)         #@UndefinedVariable
         except:
-            from xpra.log import Logger
-            log = Logger("util")
-            log.error("setting stdin to binary mode failed", exc_info=True)
+            get_util_logger().error("setting stdin to binary mode failed", exc_info=True)
 
 def find_lib_ldconfig(libname):
     libname = re.escape(libname)
@@ -529,8 +643,8 @@ def find_lib_ldconfig(libname):
             ldconfig = t
             break
     import subprocess
-    p = subprocess.Popen([ldconfig, "-p"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    data = p.communicate()[0]
+    p = subprocess.Popen([ldconfig, "-p"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
+    data = bytestostr(p.communicate()[0])
 
     libpath = re.search(pattern, data, re.MULTILINE)
     if libpath:
@@ -565,7 +679,11 @@ def pollwait(process, timeout=5):
 
 def which(command):
     from distutils.spawn import find_executable
-    return find_executable(command)
+    try:
+        return find_executable(command)
+    except Exception:
+        get_util_logger().debug("find_executable(%s)", command, exc_info=True)
+        return None
 
 def get_status_output(*args, **kwargs):
     import subprocess
@@ -597,8 +715,7 @@ def get_ssh_port():
 def setuidgid(uid, gid):
     if not POSIX:
         return
-    from xpra.log import Logger
-    log = Logger("server")
+    log = get_util_logger()
     if os.getuid()!=uid or os.getgid()!=gid:
         #find the username for the given uid:
         from pwd import getpwuid
@@ -637,15 +754,13 @@ def setuidgid(uid, gid):
     log("new uid=%s, gid=%s", os.getuid(), os.getgid())
 
 def get_peercred(sock):
-    from xpra.log import Logger
-    log = Logger("network")
     if LINUX:
         SO_PEERCRED = 17
+        log = get_util_logger()
         try:
             import socket
-            import struct
-            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize('3i'))
-            pid, uid, gid = struct.unpack('3i',creds)
+            creds = sock.getsockopt(socket.SOL_SOCKET, SO_PEERCRED, struct.calcsize(b'3i'))
+            pid, uid, gid = struct.unpack(b'3i',creds)
             log("peer: %s", (pid, uid, gid))
             return pid, uid, gid
         except Exception as  e:
@@ -660,9 +775,8 @@ def get_peercred(sock):
 
 
 def main():
-    from xpra.log import Logger
-    log = Logger("util")
     sp = sys.platform
+    log = get_util_logger()
     log.info("platform_name(%s)=%s", sp, platform_name(sp, ""))
     if LINUX:
         log.info("linux_distribution=%s", get_linux_distribution())
