@@ -6,6 +6,7 @@
 
 from xpra.keyboard.mask import mask_to_names, MODIFIER_MAP
 from xpra.log import Logger
+from xpra.os_util import bytestostr
 log = Logger("keyboard")
 
 
@@ -32,11 +33,21 @@ class KeyboardBase(object):
         self.modifier_keys = {}
         self.modifier_keycodes = {}
         for modifier, keys in mappings.items():
-            for keycode,keyname in keys:
-                self.modifier_keys[keyname] = modifier
-                keycodes = self.modifier_keycodes.setdefault(keyname, [])
-                if keycode not in keycodes:
-                    keycodes.append(keycode)
+            for a, b in keys:
+                #log.info("%s (%s), %s (%s)", keycode, type(keycode), keyname, type(keyname))
+                if isinstance(a, int) and isinstance(b, bytes):
+                    keycode = a
+                    keyname = b
+                elif isinstance(a, bytes) and isinstance(b, int):
+                    keyname = a
+                    #level = b
+                    keycode = 0
+                self.modifier_keys[bytestostr(keyname)] = bytestostr(modifier)
+                if keycode:
+                    keycodes = self.modifier_keycodes.setdefault(bytestostr(keyname), [])
+                    if keycode not in keycodes:
+                        keycodes.append(keycode)
+                self.modifier_keys[bytestostr(keyname)] = bytestostr(modifier)
         log("modifier_keys=%s", self.modifier_keys)
         log("modifier_keycodes=%s", self.modifier_keycodes)
 
@@ -57,12 +68,12 @@ class KeyboardBase(object):
         return {}
 
     def get_layout_spec(self):
-        return "", [], "", None
+        return "", [], "", None, ""
 
     def get_keyboard_repeat(self):
         return None
 
-    def update_modifier_map(self, display, xkbmap_mod_meanings):
+    def update_modifier_map(self, _display, _xkbmap_mod_meanings):
         self.modifier_map = MODIFIER_MAP
 
 
