@@ -7,6 +7,7 @@
 import os.path
 import sys
 import ctypes
+from xpra.os_util import get_util_logger
 
 shell32 = ctypes.WinDLL("shell32", use_last_error=True)
 SHGetFolderPath = shell32.SHGetFolderPathW
@@ -40,22 +41,58 @@ def _get_data_dir():
     return data_dir
 
 
-def do_get_icon_dir():
+def do_get_resources_dir():
     from xpra.platform.paths import get_app_dir
-    return os.path.join(get_app_dir(), "icons")
+    app_dir = get_app_dir()
+    prefix = os.environ.get("MINGW_PREFIX")
+    for d in (app_dir, prefix):
+        if not d or not os.path.isdir(d):
+            continue
+        share_xpra = os.path.join(d, "share", "xpra")
+        if os.path.exists(share_xpra):
+            return share_xpra
+    return app_dir
+
+def do_get_icon_dir():
+    from xpra.platform.paths import get_resources_dir
+    return os.path.join(get_resources_dir(), "icons")
 
 
-def do_get_system_conf_dirs():
-    #ie: "C:\Documents and Settings\All Users\Application Data\Xpra" with XP
-    #or: "C:\ProgramData\Xpra" with Vista onwards
+def get_program_data_dir():
+    #ie: "C:\ProgramData"
     try:
         buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
         SHGetFolderPath(0, CSIDL_COMMON_APPDATA, None, 0, buf)
         if buf.value:
-            return [os.path.join(buf.value, "Xpra")]
+            return buf.value
     except:
-        pass
-    return []
+        get_util_logger().log("get_program_data_dir()", exc_info=True)
+    return u"C:\\ProgramData"
+
+def do_get_system_conf_dirs():
+    #ie: C:\ProgramData\Xpra
+    return [os.path.join(get_program_data_dir(), u"Xpra")]
+
+def do_get_ssh_conf_dirs():
+    from xpra.scripts.config import python_platform
+    if python_platform.architecture()[0]=="32bit":
+        system32 = "SysNative"
+    else:
+        system32 = "System32"
+    windows_dir = os.environ.get("SystemRoot", os.environ.get("WINDIR", "C:\\Windows"))
+    openssh_dir = os.path.join(windows_dir, system32, "OpenSSH")
+    return [
+        os.path.join(get_program_data_dir(), "ssh"),    #ie: C:\ProgramData\ssh
+        "%APPDATA%\\ssh",   #ie: C:\Users\Username\AppData\Roaming\ssh
+        openssh_dir,        #ie: C:\Windows\system32\OpenSSH
+        "~/.ssh",
+        "~/ssh",
+        ]
+
+def do_get_ssh_known_hosts_files():
+    #reverse the order (avoid dotfiles on win32):
+    return ("~/ssh/known_hosts", "~/.ssh/known_hosts")
+
 
 def do_get_default_conf_dirs():
     #ie: C:\Program Files\Xpra\
@@ -63,8 +100,7 @@ def do_get_default_conf_dirs():
     return [os.path.join(get_app_dir(), "etc", "xpra")]
 
 def do_get_user_conf_dirs(_uid):
-    #ie: "C:\Documents and Settings\<user name>\Application Data\Xpra" with XP
-    #or: "C:\Users\<user name>\AppData\Roaming" with Visa onwards
+    #ie: "C:\Users\<user name>\AppData\Roaming"
     return [_get_data_dir()]
 
 
@@ -97,18 +133,19 @@ def do_get_script_bin_dirs():
     return []
 
 def do_get_socket_dirs():
-    #ie: C:\Documents and Settings\Username\Application Data\Xpra
-    return [_get_data_dir()]
+    return []
 
 
 APP_DIR = None
-if getattr(sys, 'frozen', False) in (True, "windows_exe", "console_exe"):    #@UndefinedVariable
+if getattr(sys, 'frozen', False) is True:
     #cx_freeze = sys.frozen == True
-    #py2exe =  sys.frozen in ("windows_exe", "console_exe")
     if sys.version_info >= (3,0):
         APP_DIR = os.path.dirname(sys.executable)
     else:
         APP_DIR = os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))
+    if len(APP_DIR)>3 and APP_DIR[1]==":" and APP_DIR[2]=="/":
+        #it seems that mingw builds can get confused about the correct value for os.pathsep:
+        APP_DIR = APP_DIR.replace("/", "\\")
     sys.path.insert(0, APP_DIR)
     os.chdir(APP_DIR)
     #so we can easily load DLLs with ctypes:
@@ -117,9 +154,6 @@ if getattr(sys, 'frozen', False) in (True, "windows_exe", "console_exe"):    #@U
     else:
         os.environ['PATH'] = APP_DIR.encode('utf8') + os.pathsep + os.environ['PATH']
 
-def do_get_resources_dir():
-    from xpra.platform.paths import get_app_dir
-    return get_app_dir()
 
 def do_get_app_dir():
     global APP_DIR
@@ -131,12 +165,22 @@ def do_get_app_dir():
 def do_get_sound_command():
     from xpra.platform.paths import get_app_dir
     app_dir = get_app_dir()
-    #is there a python3 bundled sound subdirectory
-    sound_exe = os.path.join(app_dir, "Audio", "Xpra_Audio.exe")
-    #same directory, but with a nicer name:
-    if not os.path.exists(sound_exe):
-        sound_exe = os.path.join(app_dir, "Xpra_Audio.exe")
-    #fallback for older build method:
-    if not os.path.exists(sound_exe):
-        sound_exe = os.path.join(app_dir, "xpra_cmd.exe")
-    return [sound_exe]
+    for apaths in (
+        ("Audio", "Xpra_Audio.exe"),    #python3 bundled sound subdirectory
+        ("Xpra_Audio.exe",),            #same directory, but with a nicer name:
+        ("xpra_cmd.exe",),              #fallback for older build method
+        ):
+        sound_exe = os.path.join(app_dir, *apaths)
+        if os.path.exists(sound_exe):
+            return [sound_exe]
+    return do_get_xpra_command()
+
+def do_get_xpra_command():
+    mingw = os.environ.get("MINGW_PREFIX")
+    if mingw:
+        xpra_script = os.path.join(mingw, "bin", "xpra")
+        py = os.path.join(mingw, "bin", "python%i.exe" % sys.version_info[0])
+        if os.path.exists(xpra_script) and os.path.exists(py):
+            return [py, xpra_script]
+    from xpra.platform.paths import default_do_get_xpra_command
+    return default_do_get_xpra_command()
