@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,13 +10,28 @@ import sys
 import os
 
 import xpra
-from xpra.util import updict
-from xpra.os_util import get_linux_distribution
-from xpra.log import Logger
-log = Logger("util")
+from xpra.util import updict, envbool, get_util_logger
+from xpra.os_util import get_linux_distribution, PYTHON3, BITS
 
 XPRA_VERSION = xpra.__version__     #@UndefinedVariable
 
+
+def log(msg, *args):
+    get_util_logger().debug(msg, *args)
+def warn(msg, *args):
+    get_util_logger().warn(msg, *args)
+
+
+def full_version_str():
+    s = XPRA_VERSION
+    try:
+        from xpra.src_info import REVISION, LOCAL_MODIFICATIONS
+        s += "-r%i%s" % (REVISION, ["","M"][int(LOCAL_MODIFICATIONS>0)])
+    except:
+        pass
+    return s
+
+
 def version_as_numbers(version):
     return [int(x) for x in version.split(".")]
 
@@ -46,15 +61,13 @@ def get_host_info():
     #this function is for non UI thread info
     info = {}
     try:
-        import struct
-        bits = struct.calcsize("P") * 8
         import socket
         info.update({
                     "pid"                   : os.getpid(),
                     "byteorder"             : sys.byteorder,
                     "hostname"              : socket.gethostname(),
                     "python"                : {
-                                               "bits"                  : bits,
+                                               "bits"                  : BITS,
                                                "full_version"          : sys.version,
                                                "version"               : ".".join(str(x) for x in sys.version_info[:3]),
                                                },
@@ -78,7 +91,7 @@ def get_version_info():
         props["local_modifications"]    = LOCAL_MODIFICATIONS
         props["revision"]               = REVISION
     except ImportError as e:
-        log.warn("missing some source information: %s", e)
+        warn("missing some source information: %s", e)
     return props
 
 def get_version_info_full():
@@ -105,7 +118,7 @@ def get_version_info_full():
         d = dict((k.lstrip("lib_"), getattr(build_info, k)) for k in dir(build_info) if k.startswith("lib_"))
         updict(props, "lib", d)
     except Exception as e:
-        log.warn("missing some build information: %s", e)
+        warn("missing some build information: %s", e)
     log("get_version_info_full()=%s", props)
     return props
 
@@ -155,3 +168,62 @@ def get_platform_info():
     if platform_info_cache is None:
         platform_info_cache = do_get_platform_info()
     return platform_info_cache
+
+
+def get_version_from_url(url):
+    e = None
+    try:
+        if PYTHON3:
+            from urllib.request import urlopen
+        else:
+            from urllib2 import urlopen
+    except ImportError:
+        log("get_version_from_url(%s) urllib2 not found: %s", url, e)
+        return None
+    try:
+        response = urlopen(url)
+        latest_version = response.read().rstrip(b"\n\r")
+        latest_version_no = tuple(int(y) for y in latest_version.split(b"."))
+        log("get_version_from_url(%s)=%s", url, latest_version_no)
+        return latest_version_no
+    except Exception as e:
+        log("get_version_from_url(%s)", url, exc_info=True)
+        if hasattr(e, "code") and e.code==404:
+            log("no version at url=%s", url)
+        else:
+            log("Error retrieving URL '%s': %s", url, e)
+    return None
+
+def version_update_check():
+    FAKE_NEW_VERSION = envbool("XPRA_FAKE_NEW_VERSION", False)
+    from xpra.os_util import WIN32
+    if WIN32:
+        #no idea how to make it verify the certificate properly
+        #so don't bother
+        CURRENT_VERSION_URL = "http://xpra.org/CURRENT_VERSION"
+    else:
+        CURRENT_VERSION_URL = "https://xpra.org/CURRENT_VERSION"
+    PLATFORM_FRIENDLY_NAMES = {
+        "linux2"    : "LINUX",
+        "win"       : "WINDOWS",
+        "darwin"    : "OSX",
+        }
+    our_version_no = tuple(int(y) for y in XPRA_VERSION.split("."))
+    platform_name = PLATFORM_FRIENDLY_NAMES.get(sys.platform, sys.platform)
+    arch = get_platform_info().get("machine")
+    latest_version_no = None
+    if arch:
+        latest_version_no = get_version_from_url("%s_%s_%s?%s" % (CURRENT_VERSION_URL, platform_name, arch, XPRA_VERSION))
+    if not latest_version_no:
+        latest_version_no = get_version_from_url("%s_%s?%s" % (CURRENT_VERSION_URL, platform_name, XPRA_VERSION))
+    if not latest_version_no:
+        latest_version_no = get_version_from_url("%s?%s" % (CURRENT_VERSION_URL, XPRA_VERSION))
+    if latest_version_no is None:
+        log("version_update_check() failed to contact version server")
+        return None
+    elif latest_version_no>our_version_no or FAKE_NEW_VERSION:
+        log("version_update_check() newer version found! local version is %s and the latest version available is %s", our_version_no, latest_version_no)
+        #latest_version = ".".join([str(x) for x in latest_version_no])
+        return latest_version_no
+    else:
+        return False
