@@ -18,6 +18,7 @@ from collections import deque, namedtuple
 from xpra.version_util import XPRA_VERSION
 from xpra.util import updict, rindex, envbool, envint
 from xpra.os_util import memoryview_to_bytes, monotonic_time
+from xpra.server import server_features
 from xpra.gtk_common.gobject_util import one_arg_signal
 from xpra.gtk_common.gtk_util import get_default_root_window, get_xwindow
 from xpra.x11.common import Unmanageable
@@ -190,6 +191,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self.configure_damage_timers = {}
         self._tray = None
         self.system_tray = False
+        self.global_menus = False
         gobject.GObject.__init__(self)
         X11ServerBase.__init__(self)
         self.session_type = "seamless"
@@ -197,12 +199,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
     def init(self, opts):
         self.wm_name = opts.wm_name
         self.sync_xvfb = int(opts.sync_xvfb)
-        self.global_menus = int(opts.global_menus)
         self.system_tray = opts.system_tray
         X11ServerBase.init(self, opts)
         self.fake_xinerama = opts.fake_xinerama
-        if self.global_menus:
-            self.rpc_handlers["menu"] = self._handle_menu_rpc
+        #rpc_handlers are part of DBUS_RPC_Server, which is optional
+        rpc_handlers = getattr(self, "rpc_handlers", None)
+        if rpc_handlers:
+            self.global_menus = int(opts.global_menus)
+            if self.global_menus:
+                self.rpc_handlers["menu"] = self._handle_menu_rpc
         def log_server_event(_, event):
             eventlog("server-event: %s", event)
         self.connect("server-event", log_server_event)
@@ -235,17 +240,23 @@ class XpraServer(gobject.GObject, X11ServerBase):
                     self.root_overlay = X11Window.XCompositeGetOverlayWindow(root.xid)
                     if self.root_overlay:
                         #ugly: API expects a window object with a ".xid"
-                        X11WindowModel = namedtuple("X11-WindowModel", "xid")
+                        X11WindowModel = namedtuple("X11WindowModel", "xid")
                         root_overlay = X11WindowModel(xid=self.root_overlay)
                         prop_set(root_overlay, "WM_TITLE", "latin1", u"RootOverlay")
                         X11Window.AllowInputPassthrough(self.root_overlay)
             except Exception as e:
+                log("XCompositeGetOverlayWindow(%#x)", root.xid, exc_info=True)
                 log.error("Error setting up xvfb synchronization:")
                 log.error(" %s", e)
+                if self.root_overlay:
+                    with xswallow:
+                        X11Window.XCompositeReleaseOverlayWindow(self.root_overlay)
+                    self.root_overlay = None
 
         ### Create the WM object
         self._wm = Wm(self.clobber, self.wm_name)
-        self._wm.connect("new-window", self._new_window_signaled)
+        if server_features.windows:
+            self._wm.connect("new-window", self._new_window_signaled)
         self._wm.connect("quit", lambda _: self.clean_quit(True))
         self._wm.connect("show-desktop", self._show_desktop)
 
@@ -311,6 +322,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 "resize-counter"         : True,
                 "configure.skip-geometry": True,
                 "configure.pointer"      : True,
+                "configure.delta"        : True,
                 "signals"                : WINDOW_SIGNALS,
                 "dragndrop"              : True,
                 "states"                 : ["iconified", "fullscreen", "above", "below", "sticky", "iconified", "maximized"],
@@ -321,8 +333,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
     ##########################################################################
     # info:
     #
-    def do_get_info(self, proto, server_sources, window_ids):
-        info = X11ServerBase.do_get_info(self, proto, server_sources, window_ids)
+    def do_get_info(self, proto, server_sources):
+        info = X11ServerBase.do_get_info(self, proto, server_sources)
         info.setdefault("state", {}).update({
                                              "focused"  : self._has_focus,
                                              "grabbed"  : self._has_grab,
@@ -360,7 +372,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         #when we have clients, this should have been done already
         #in the code that synchonizes the screen resolution
         if len(self._server_sources)==0:
-            X11ServerBase.set_screen_geometry_attributes(self, w, h)
+            super(XpraServer, self).set_screen_geometry_attributes(w, h)
 
     def set_desktops(self, names):
         wm = self._wm
@@ -444,7 +456,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         framelog("parse_hello_ui_window_settings: client window_frame_sizes=%s", ss.window_frame_sizes)
         frame = None
         if ss.window_frame_sizes:
-            frame = ss.window_frame_sizes.intlistget("frame")
+            frame = ss.window_frame_sizes.intlistget("frame", (0, 0, 0, 0), 4, 4)
         self._wm.set_default_frame_extents(frame)
 
 
@@ -497,7 +509,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def _add_new_window_common(self, window):
         windowlog("adding window %s", window)
-        wid = X11ServerBase._add_new_window_common(self, window)
+        wid = super(XpraServer, self)._add_new_window_common(window)
         window.managed_connect("client-contents-changed", self._contents_changed)
         window.managed_connect("unmanaged", self._lost_window)
         window.managed_connect("grab", self._window_grab)
@@ -507,10 +519,10 @@ class XpraServer(gobject.GObject, X11ServerBase):
         if not window.is_tray():
             window.managed_connect("raised", self._raised_window)
             window.managed_connect("initiate-moveresize", self._initiate_moveresize)
-        prop_set = getattr(window, "prop_set", None)
-        if prop_set:
+        window_prop_set = getattr(window, "prop_set", None)
+        if window_prop_set:
             try:
-                prop_set("_XPRA_WID", "u32", wid)
+                window_prop_set("_XPRA_WID", "u32", wid)
             except:
                 pass    #this can fail if the window disappears
                 #but we don't really care, it will get cleaned up soon enough
@@ -636,6 +648,21 @@ class XpraServer(gobject.GObject, X11ServerBase):
             ss.or_window_geometry(wid, window, x, y, w, h)
 
 
+    def add_control_commands(self):
+        super(XpraServer, self).add_control_commands()
+        from xpra.server.control_command import ArgsControlCommand
+        cmd = ArgsControlCommand("show-all-windows", "make all the windows visible", validation=[])
+        def control_cb():
+            self.show_all_windows()
+            return "%i windows shown" % len(self._id_to_window)
+        cmd.do_run = control_cb
+        self.control_commands[cmd.name] = cmd
+
+    def show_all_windows(self):
+        for w in self._id_to_window.values():
+            self._desktop_manager.show_window(w)
+
+
     def _show_desktop(self, wm, show):
         log("show_desktop(%s, %s)", wm, show)
         for ss in self._server_sources.values():
@@ -654,7 +681,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
             if self._wm:
                 toplevel = self._wm.get_property("toplevel")
             focuslog("reset_focus() %s / %s had focus (toplevel=%s)", self._has_focus, had_focus, toplevel)
-            self.clear_keys_pressed()
+            #this will call clear_keys_pressed() if the server is an InputServer:
+            self.reset_focus()
             # FIXME: kind of a hack:
             self._has_focus = 0
             #toplevel may be None during cleanup!
@@ -678,8 +706,10 @@ class XpraServer(gobject.GObject, X11ServerBase):
         window.raise_window()
         window.give_client_focus()
         if server_source and modifiers is not None:
-            focuslog("focus: will set modified mask to %s", modifiers)
-            server_source.make_keymask_match(modifiers)
+            make_keymask_match = getattr(server_source, "make_keymask_match", None)
+            if make_keymask_match:
+                focuslog("focus: will set modified mask to %s using %s", modifiers, make_keymask_match)
+                make_keymask_match(modifiers)
         self._has_focus = wid
 
     def get_focus(self):
@@ -805,8 +835,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self._window_mapped_at(proto, wid, window, (x, y, w, h))
         if len(packet)>=7:
             self._set_client_properties(proto, wid, window, packet[6])
-        if not self.ui_driver:
-            self.ui_driver = ss.uuid
+        #if not self.ui_driver:
+        #    self.set_ui_driver(ss)
         if self.ui_driver==ss.uuid or not self._desktop_manager.is_shown(window):
             if len(packet)>=8:
                 self._set_window_state(proto, wid, window, packet[7])
@@ -826,11 +856,9 @@ class XpraServer(gobject.GObject, X11ServerBase):
         ss = self._server_sources.get(proto)
         if ss is None:
             return
-        self._window_mapped_at(proto, wid, window, None)
-        if not self.ui_driver:
-            self.ui_driver = ss.uuid
-        elif self.ui_driver!=ss.uuid:
-            return
+        self._window_mapped_at(proto, wid, window)
+        #if self.ui_driver!=ss.uuid:
+        #    return
         if len(packet)>=4:
             #optional window_state added in 0.15 to update flags
             #during iconification events:
@@ -880,8 +908,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
             if cprops:
                 metadatalog("window client properties updates: %s", cprops)
                 self._set_client_properties(proto, wid, window, cprops)
-        if not self.ui_driver:
-            self.ui_driver = ss.uuid
+        #if not self.ui_driver:
+        #    self.set_ui_driver(ss)
         is_ui_driver = self.ui_driver==ss.uuid
         shown = self._desktop_manager.is_shown(window)
         if window.is_OR() or window.is_tray() or skip_geometry:
@@ -891,14 +919,14 @@ class XpraServer(gobject.GObject, X11ServerBase):
             size_changed = oww!=w or owh!=h
         if is_ui_driver or size_changed or not shown:
             damage = False
-            if is_ui_driver and len(packet)>=13 and not self.readonly:
+            if is_ui_driver and len(packet)>=13 and server_features.input_devices and not self.readonly:
                 pwid = packet[10]
                 pointer = packet[11]
                 modifiers = packet[12]
-                #only update modifiers if the window is in focus:
-                if self._has_focus==wid:
-                    self._update_modifiers(proto, wid, modifiers)
-                self._process_mouse_common(proto, pwid, pointer)
+                if self._process_mouse_common(proto, pwid, pointer):
+                    #only update modifiers if the window is in focus:
+                    if self._has_focus==wid:
+                        self._update_modifiers(proto, wid, modifiers)
             if window.is_tray():
                 assert self._tray
                 if not skip_geometry:
@@ -1035,7 +1063,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
 
     def _damage(self, window, x, y, width, height, options=None):
-        X11ServerBase._damage(self, window, x, y, width, height, options)
+        super(XpraServer, self)._damage(window, x, y, width, height, options)
         if self.root_overlay:
             image = window.get_image(x, y, width, height)
             if image:
@@ -1188,14 +1216,16 @@ class XpraServer(gobject.GObject, X11ServerBase):
                     else:
                         cr.stroke()
         #FIXME: use server mouse position, and use current cursor shape
-        if ss and ss.mouse_last_position and ss.mouse_last_position!=(0, 0):
-            x, y = ss.mouse_last_position
-            cr.set_source_rgb(1.0, 0.5, 0.7)
-            cr.new_path()
-            cr.arc(x, y, 10.0, 0, 2.0 * math.pi)
-            cr.stroke_preserve()
-            cr.set_source_rgb(0.3, 0.4, 0.6)
-            cr.fill()
+        if ss:
+            mlp = getattr(ss, "mouse_last_position", (0, 0))
+            if mlp!=(0, 0):
+                x, y = mlp
+                cr.set_source_rgb(1.0, 0.5, 0.7)
+                cr.new_path()
+                cr.arc(x, y, 10.0, 0, 2.0 * math.pi)
+                cr.stroke_preserve()
+                cr.set_source_rgb(0.3, 0.4, 0.6)
+                cr.fill()
         return False
 
 
